#
# Copyright (C) 2014-2018 Pico Technology Ltd. See LICENSE file for terms.
#
"""
This is a Python module defining the functions from the ps6000Api.h C header
file for PicoScope 6000 Series oscilloscopes using the ps6000 driver API
functions.
"""

from ctypes import *
from picosdk.library import Library
from picosdk.ctypes_wrapper import C_CALLBACK_FUNCTION_FACTORY
from picosdk.constants import make_enum


class Ps6000lib(Library):
    def __init__(self):
        super(Ps6000lib, self).__init__("ps6000")


ps6000 = Ps6000lib()

ps6000.PS6000_RANGE = make_enum([
    "PS6000A_10MV",
    "PS6000_20MV",
    "PS6000_50MV",
    "PS6000_100MV",
    "PS6000_200MV",
    "PS6000_500MV",
    "PS6000_1V",
    "PS6000_2V",
    "PS6000_5V",
    "PS6000_10V",
    "PS6000_20V",
    "PS6000_50V",
    "PS6000_MAX_RANGES",
])

ps6000.PS6000_CHANNEL = make_enum([
	"PS6000_CHANNEL_A",
	"PS6000_CHANNEL_B",
	"PS6000_CHANNEL_C",
	"PS6000_CHANNEL_D",
	("PS6000_EXTERNAL", "PS6000_MAX_CHANNELS"),
	"PS6000_TRIGGER_AUX",
	"PS6000_MAX_TRIGGER_SOURCES",
])

ps6000.PS6000_COUPLING = make_enum([
	"PS6000_AC",
	"PS6000_DC_1M",
	"PS6000_DC_50R",
])

ps6000.PS6000_BANDWIDTH_LIMITER = make_enum([
	"PS6000_BW_FULL",
	"PS6000_BW_20MHZ",
	"PS6000_BW_25MHZ",
])

ps6000.PS6000_RATIO_MODE = {
	"PS6000_RATIO_MODE_NONE" : 0,
	"PS6000_RATIO_MODE_AGGREGATE" : 1,
	"PS6000_RATIO_MODE_AVERAGE" : 2,
	"PS6000_RATIO_MODE_DECIMATE" : 4,
	"PS6000_RATIO_MODE_DISTRIBUTION" : 8,
}

ps6000.PS6000_TIME_UNITS = make_enum([
	"PS6000_FS",
	"PS6000_PS",
	"PS6000_NS",
	"PS6000_US",
	"PS6000_MS",
	"PS6000_S",
	"PS6000_MAX_TIME_UNITS",
])

ps6000.PS6000_TRIGGER_STATE = make_enum([
	"PS6000_CONDITION_DONT_CARE",
	"PS6000_CONDITION_TRUE",
	"PS6000_CONDITION_FALSE",
	"PS6000_CONDITION_MAX",
])

ps6000.PS6000_THRESHOLD_DIRECTION = make_enum([
	("PS6000_ABOVE", "PS6000_INSIDE"),
	("PS6000_BELOW", "PS6000_BELOW"),
	("PS6000_RISING", "PS6000_ENTER", "PS6000_NONE"),
	("PS6000_FALLING", "PS6000_EXIT"),
	("PS6000_RISING_OR_FALLING", "PS6000_ENTER_OR_EXIT"),
	"PS6000_ABOVE_LOWER",
	"PS6000_BELOW_LOWER",
	"PS6000_RISING_LOWER",
	"PS6000_FALLING_LOWER",
	"PS6000_POSITIVE_RUNT",
	"PS6000_NEGATIVE_RUNT",
])
	
ps6000.PS6000_THRESHOLD_MODE = make_enum([
	"PS6000_LEVEL",
	"PS6000_WINDOW",
])

ps6000.PS6000_PULSE_WIDTH_TYPE = make_enum([
	"PS6000_PW_TYPE_NONE",
	"PS6000_PW_TYPE_LESS_THAN",
	"PS6000_PW_TYPE_GREATER_THAN",
	"PS6000_PW_TYPE_IN_RANGE",
	"PS6000_PW_TYPE_OUT_OF_RANGE"
])

ps6000.PICO_INFO = {
    "PICO_DRIVER_VERSION" : 0,
    "PICO_USB_VERSION" : 1,
    "PICO_HARDWARE_VERSION" : 2,
    "PICO_VARIANT_INFO" : 3,
    "PICO_BATCH_AND_SERIAL" : 4,
    "PICO_CAL_DATE" : 5,
    "PICO_KERNAL_VERSION" : 6,
    "PICO_DIGITAL_HARDWARE_VERSION" : 7,
    "PICO_ANALOGUE_HARDWARE_VERSION" : 8,
    "PICO_FIRMWARE_VERSION_1" : 9,
    "PICO_FIREWARE_VERSION_2" : 'A'
}
    

class PS6000_TRIGGER_CONDITIONS (Structure):
    _pack_ = 1
    _fields_ = [("channelA", c_uint32),
                ("channelB", c_uint32),
				("channelC", c_uint32),
				("channelD", c_uint32),
				("external", c_uint32),
				("aux", c_uint32),
				("pulseWidthQualifier", c_uint32)]
				
ps6000.PS6000_TRIGGER_CONDITIONS = PS6000_TRIGGER_CONDITIONS

class PS6000_TRIGGER_CHANNEL_PROPERTIES (Structure):
    _pack_ = 1
    _fields_ = [("thresholdUpper", c_int16),
                ("hysteresisUpper", c_uint16),
                ("thresholdLower", c_int16),
                ("hysteresisLower", c_uint16),
                ("channel", c_uint32),
                ("thresholdMode", c_uint32)]
				
ps6000.PS6000_TRIGGER_CHANNEL_PROPERTIES = PS6000_TRIGGER_CHANNEL_PROPERTIES

class PS6000_PWQ_CONDITIONS (Structure):
	_pack_ = 1
	_fields_ = [("channelA", c_uint32),
				("channelB", c_uint32),
				("channelC", c_uint32),
				("channelD", c_uint32),
				("external", c_uint32),
				("aux", c_uint32)]
				
ps6000.PS6000_PWQ_CONDITIONS = PS6000_PWQ_CONDITIONS

class PS6000_TRIGGER_INFO (Structure):
    _pack_ = 1
    _fields_ = [("status", c_uint32),
                ("segmentIndex", c_uint32),
                ("triggerIndex", c_uint32),
                ("triggerTime", c_uint64),
                ("timeUnits", c_uint16),
                ("reserved0", c_uint16),
                ("timeStampCounter", c_uint64)]
                
ps6000.PS6000_TRIGGER_INFO = PS6000_TRIGGER_INFO
	


# some ps6000 scopes have 2GS of memory.
ps6000.MAX_MEMORY = 2**31

doc = """ PICO_STATUS ps6000OpenUnit
    (
        int16_t *handle,
        int8_t  *serial
    ); """
ps6000.make_symbol("_OpenUnit", "ps6000OpenUnit", c_uint32, [c_void_p, c_char_p], doc)

doc = """ PICO_STATUS ps6000OpenUnitAsync
    (
        int16_t *status,
        int8_t  *serial
    ); """
ps6000.make_symbol("_OpenUnitAsync", "ps6000OpenUnitAsync", c_uint32, [c_void_p, c_char_p], doc)

doc = """ PICO_STATUS ps6000OpenUnitProgress
    (
      int16_t *handle,
      int16_t *progressPercent,
      int16_t *complete
    ); """
ps6000.make_symbol("_OpenUnitProgress", "ps6000OpenUnitProgress", c_uint32, [c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000GetUnitInfo
    (
        int16_t    handle,
        int8_t    *string,
        int16_t    stringLength,
        int16_t   *requiredSize,
        PICO_INFO  info
    ); """
ps6000.make_symbol("_GetUnitInfo", "ps6000GetUnitInfo", c_uint32, [c_int16, c_void_p, c_int16, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps6000FlashLed
    (
        int16_t  handle,
        int16_t  start
    ); """
ps6000.make_symbol("_FlashLed", "ps6000FlashLed", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps6000CloseUnit
    (
        int16_t  handle
    ); """
ps6000.make_symbol("_CloseUnit", "ps6000CloseUnit", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps6000MemorySegments
    (
        int16_t   handle,
        uint32_t  nSegments,
        uint32_t *nMaxSamples
    ); """
ps6000.make_symbol("_MemorySegments", "ps6000MemorySegments", c_uint32, [c_int16, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps6000SetChannel
    (
        int16_t                   handle,
        PS6000_CHANNEL            channel,
        int16_t                   enabled,
        PS6000_COUPLING           type,
        PS6000_RANGE              range,
        float                     analogueOffset,
        PS6000_BANDWIDTH_LIMITER  bandwidth
    ); """
ps6000.make_symbol("_SetChannel", "ps6000SetChannel", c_uint32,
                   [c_int16, c_int32, c_int16, c_int32, c_int32, c_float, c_int32], doc)

doc = """ PICO_STATUS ps6000GetTimebase
    (
        int16_t   handle,
        uint32_t  timebase,
        uint32_t  noSamples,
        int32_t  *timeIntervalNanoseconds,
        int16_t   oversample,
        uint32_t *maxSamples,
        uint32_t  segmentIndex
    ); """
ps6000.make_symbol("_GetTimebase", "ps6000GetTimebase", c_uint32,
                   [c_int16, c_uint32, c_uint32, c_void_p, c_int16, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps6000GetTimebase2
    (
        int16_t   handle,
        uint32_t  timebase,
        uint32_t  noSamples,
        float    *timeIntervalNanoseconds,
        int16_t   oversample,
        uint32_t *maxSamples,
        uint32_t  segmentIndex
    ); """
ps6000.make_symbol("_GetTimebase2", "ps6000GetTimebase2", c_uint32,
                   [c_int16, c_uint32, c_uint32, c_void_p, c_int16, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps6000SetSigGenArbitrary
    (
        int16_t                    handle,
        int32_t                    offsetVoltage,
        uint32_t                   pkToPk,
        uint32_t                   startDeltaPhase,
        uint32_t                   stopDeltaPhase,
        uint32_t                   deltaPhaseIncrement,
        uint32_t                   dwellCount,
        int16_t                   *arbitraryWaveform,
        int32_t                    arbitraryWaveformSize,
        PS6000_SWEEP_TYPE          sweepType,
        PS6000_EXTRA_OPERATIONS    operation,
        PS6000_INDEX_MODE          indexMode,
        uint32_t                   shots,
        uint32_t                   sweeps,
        PS6000_SIGGEN_TRIG_TYPE    triggerType,
        PS6000_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                    extInThreshold
    ); """
ps6000.make_symbol("_SetSigGenArbitrary", "ps6000SetSigGenArbitrary", c_uint32,
                   [c_int16, c_int32, c_uint32, c_uint32, c_uint32, c_uint32, c_uint32, c_void_p,
                    c_int32, c_int32, c_int32, c_int32, c_uint32, c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps6000SetSigGenBuiltIn
    (
        int16_t                    handle,
        int32_t                    offsetVoltage,
        uint32_t                   pkToPk,
        int16_t                    waveType,
        float                      startFrequency,
        float                      stopFrequency,
        float                      increment,
        float                      dwellTime,
        PS6000_SWEEP_TYPE          sweepType,
        PS6000_EXTRA_OPERATIONS    operation,
        uint32_t                   shots,
        uint32_t                   sweeps,
        PS6000_SIGGEN_TRIG_TYPE    triggerType,
        PS6000_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                    extInThreshold
    ); """
ps6000.make_symbol("_SetSigGenBuiltIn", "ps6000SetSigGenBuiltIn", c_uint32,
                   [c_int16, c_int32, c_uint32, c_int16, c_float, c_float, c_float, c_float,
                    c_int32, c_int32, c_uint32, c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps6000SetSigGenBuiltInV2
    (
        int16_t                    handle,
        int32_t                    offsetVoltage,
        uint32_t                   pkToPk,
        int16_t                    waveType,
        double                     startFrequency,
        double                     stopFrequency,
        double                     increment,
        double                     dwellTime,
        PS6000_SWEEP_TYPE          sweepType,
        PS6000_EXTRA_OPERATIONS    operation,
        uint32_t                   shots,
        uint32_t                   sweeps,
        PS6000_SIGGEN_TRIG_TYPE    triggerType,
        PS6000_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                    extInThreshold
    ); """
ps6000.make_symbol("_SetSigGenBuiltInV2", "ps6000SetSigGenBuiltInV2", c_uint32,
                   [c_int16, c_int32, c_uint32, c_int16, c_double, c_double, c_double, c_double,
                    c_int32, c_int32, c_uint32, c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps6000SetSigGenPropertiesArbitrary
    (
        int16_t                    handle,
        int32_t                    offsetVoltage,
        uint32_t                   pkToPk,
        uint32_t                   startDeltaPhase,
        uint32_t                   stopDeltaPhase,
        uint32_t                   deltaPhaseIncrement,
        uint32_t                   dwellCount,
        PS6000_SWEEP_TYPE          sweepType,
        uint32_t                   shots,
        uint32_t                   sweeps,
        PS6000_SIGGEN_TRIG_TYPE    triggerType,
        PS6000_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                    extInThreshold
    ); """
ps6000.make_symbol("_SigGenPropertiesArbitrary", "ps6000SetSigGenPropertiesArbitrary", c_uint32,
                   [c_int16, c_int32, c_uint32, c_uint32, c_uint32, c_uint32, c_uint32,
                    c_int32, c_uint32, c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps6000SetSigGenPropertiesBuiltIn
    (
        int16_t                    handle,
        int32_t                    offsetVoltage,
        uint32_t                   pkToPk,
        double                     startFrequency,
        double                     stopFrequency,
        double                     increment,
        double                     dwellTime,
        PS6000_SWEEP_TYPE          sweepType,
        uint32_t                   shots,
        uint32_t                   sweeps,
        PS6000_SIGGEN_TRIG_TYPE    triggerType,
        PS6000_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                    extInThreshold
    ); """
ps6000.make_symbol("_SetSigGenPropertiesBuiltIn", "ps6000SetSigGenPropertiesBuiltIn", c_uint32,
                   [c_int16, c_int32, c_uint32, c_double, c_double, c_double, c_double,
                    c_int32, c_uint32, c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps6000SigGenFrequencyToPhase
    (
        int16_t            handle,
        double             frequency,
        PS6000_INDEX_MODE  indexMode,
        uint32_t           bufferLength,
        uint32_t          *phase
    ); """
ps6000.make_symbol("_SigGenFrequencyToPhase", "ps6000SigGenFrequencyToPhase", c_uint32,
                   [c_int16, c_double, c_int32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps6000SigGenArbitraryMinMaxValues
    (
        int16_t   handle,
        int16_t  *minArbitraryWaveformValue,
        int16_t  *maxArbitraryWaveformValue,
        uint32_t *minArbitraryWaveformSize,
        uint32_t *maxArbitraryWaveformSize
    ); """
ps6000.make_symbol("_SigGenArbitraryMinMaxValues", "ps6000SigGenArbitraryMinMaxValues", c_uint32,
                   [c_int16, c_void_p, c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000SigGenSoftwareControl
    (
        int16_t  handle,
        int16_t  state
    ); """
ps6000.make_symbol("_SigGenSoftwareControl", "ps6000SigGenSoftwareControl", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps6000SetSimpleTrigger
    (
        int16_t                     handle,
        int16_t                     enable,
        PS6000_CHANNEL              source,
        int16_t                     threshold,
        PS6000_THRESHOLD_DIRECTION  direction,
        uint32_t                    delay,
        int16_t                     autoTrigger_ms
    ); """
ps6000.make_symbol("_SetSimpleTrigger", "ps6000SetSimpleTrigger", c_uint32,
                   [c_int16, c_int16, c_int32, c_int16, c_int32, c_uint32, c_int16], doc)

doc = """ PICO_STATUS ps6000SetEts
    (
        int16_t          handle,
        PS6000_ETS_MODE  mode,
        int16_t          etsCycles,
        int16_t          etsInterleave,
        int32_t         *sampleTimePicoseconds
    ); """
ps6000.make_symbol("_SetEts", "ps6000SetEts", c_uint32, [c_int16, c_int32, c_int16, c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps6000SetTriggerChannelProperties
    (
        int16_t                            handle,
        PS6000_TRIGGER_CHANNEL_PROPERTIES *channelProperties,
        int16_t                            nChannelProperties,
        int16_t                            auxOutputEnable,
        int32_t                            autoTriggerMilliseconds
    ); """
ps6000.make_symbol("_SetTriggerChannelProperties", "ps6000SetTriggerChannelProperties", c_uint32,
                   [c_int16, c_void_p, c_int16, c_int16, c_int32], doc)

doc = """ PICO_STATUS ps6000SetTriggerChannelConditions
    (
        int16_t                    handle,
        PS6000_TRIGGER_CONDITIONS *conditions,
        int16_t                    nConditions
    ); """
ps6000.make_symbol("_SetTriggerChannelConditions", "ps6000SetTriggerChannelConditions", c_uint32,
                   [c_int16, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps6000SetTriggerChannelDirections
    (
        int16_t                       handle,
        PS6000_THRESHOLD_DIRECTION  channelA,
        PS6000_THRESHOLD_DIRECTION  channelB,
        PS6000_THRESHOLD_DIRECTION  channelC,
        PS6000_THRESHOLD_DIRECTION  channelD,
        PS6000_THRESHOLD_DIRECTION  ext,
        PS6000_THRESHOLD_DIRECTION  aux
    ); """
ps6000.make_symbol("_SetTriggerChannelDirections", "ps6000SetTriggerChannelDirections", c_uint32,
                   [c_int16, c_int32, c_int32, c_int32, c_int32, c_int32, c_int32], doc)

doc = """ PICO_STATUS ps6000SetTriggerDelay
    (
        int16_t   handle,
        uint32_t  delay
    ); """
ps6000.make_symbol("_SetTriggerDelay", "ps6000SetTriggerDelay", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps6000SetPulseWidthQualifier
    (
        int16_t                     handle,
        PS6000_PWQ_CONDITIONS      *conditions,
        int16_t                     nConditions,
        PS6000_THRESHOLD_DIRECTION  direction,
        uint32_t                    lower,
        uint32_t                    upper,
        PS6000_PULSE_WIDTH_TYPE     type
    ); """
ps6000.make_symbol("_SetPulseWidthQualifier", "ps6000SetPulseWidthQualifier", c_uint32,
                   [c_int16, c_void_p, c_int16, c_int32, c_uint32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps6000IsTriggerOrPulseWidthQualifierEnabled
    (
        int16_t  handle,
        int16_t *triggerEnabled,
        int16_t *pulseWidthQualifierEnabled
    ); """
ps6000.make_symbol("_IsTriggerOrPulseWidthQualifierEnabled", "ps6000IsTriggerOrPulseWidthQualifierEnabled", c_uint32,
                   [c_int16, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000GetTriggerTimeOffset
    (
        int16_t            handle,
        uint32_t          *timeUpper,
        uint32_t          *timeLower,
        PS6000_TIME_UNITS *timeUnits,
        uint32_t           segmentIndex
    ); """
ps6000.make_symbol("_GetTriggerTimeOffset", "ps6000GetTriggerTimeOffset", c_uint32,
                   [c_int16, c_void_p, c_void_p, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps6000GetTriggerTimeOffset64
    (
        int16_t              handle,
        int64_t           *time,
        PS6000_TIME_UNITS *timeUnits,
        uint32_t      segmentIndex
    ); """
ps6000.make_symbol("_GetTriggerTimeOffset64", "ps6000GetTriggerTimeOffset64", c_uint32,
                   [c_int16, c_void_p, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps6000GetValuesTriggerTimeOffsetBulk
    (
        int16_t            handle,
        uint32_t          *timesUpper,
        uint32_t          *timesLower,
        PS6000_TIME_UNITS *timeUnits,
        uint32_t           fromSegmentIndex,
        uint32_t           toSegmentIndex
    ); """
ps6000.make_symbol("_ps6000GetValuesTriggerTimeOffsetBulk", "ps6000GetValuesTriggerTimeOffsetBulk", c_uint32,
                   [c_int16, c_void_p, c_void_p, c_void_p, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS ps6000GetValuesTriggerTimeOffsetBulk64
    (
        int16_t            handle,
        int64_t           *times,
        PS6000_TIME_UNITS *timeUnits,
        uint32_t           fromSegmentIndex,
        uint32_t           toSegmentIndex
    ); """
ps6000.make_symbol("_GetValuesTriggerTimeOffsetBulk64", "ps6000GetValuesTriggerTimeOffsetBulk64", c_uint32,
                   [c_int16, c_void_p, c_void_p, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS ps6000SetDataBuffers
    (
        int16_t            handle,
        PS6000_CHANNEL     channel,
        int16_t           *bufferMax,
        int16_t           *bufferMin,
        uint32_t           bufferLth,
        PS6000_RATIO_MODE  downSampleRatioMode
    ); """
ps6000.make_symbol("_SetDataBuffers", "ps6000SetDataBuffers", c_uint32,
                   [c_int16, c_int32, c_void_p, c_void_p, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps6000SetDataBuffer
    (
        int16_t            handle,
        PS6000_CHANNEL     channel,
        int16_t           *buffer,
        uint32_t           bufferLth,
        PS6000_RATIO_MODE  downSampleRatioMode
    ); """
ps6000.make_symbol("_SetDataBuffer", "ps6000SetDataBuffer", c_uint32, [c_int16, c_int32, c_void_p, c_uint32, c_int32],
                   doc)

doc = """ PICO_STATUS (ps6000SetDataBufferBulk)
    (
        int16_t            handle,
        PS6000_CHANNEL     channel,
        int16_t           *buffer,
        uint32_t           bufferLth,
        uint32_t           waveform,
        PS6000_RATIO_MODE  downSampleRatioMode
    ); """
ps6000.make_symbol("_SetDataBufferBulk", "ps6000SetDataBufferBulk", c_uint32,
                   [c_int16, c_int32, c_void_p, c_uint32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps6000SetDataBuffersBulk
    (
        int16_t            handle,
        PS6000_CHANNEL     channel,
        int16_t           *bufferMax,
        int16_t           *bufferMin,
        uint32_t           bufferLth,
        uint32_t           waveform,
        PS6000_RATIO_MODE  downSampleRatioMode
    ); """
ps6000.make_symbol("_SetDataBuffersBulk", "ps6000SetDataBuffersBulk", c_uint32,
                   [c_int16, c_int32, c_void_p, c_void_p, c_uint32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps6000SetEtsTimeBuffer
    (
        int16_t   handle,
        int64_t  *buffer,
        uint32_t  bufferLth
    ); """
ps6000.make_symbol("_SetEtsTimeBuffer", "ps6000SetEtsTimeBuffer", c_uint32, [c_int16, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps6000SetEtsTimeBuffers
    (
        int16_t   handle,
        uint32_t *timeUpper,
        uint32_t *timeLower,
        uint32_t  bufferLth
    ); """
ps6000.make_symbol("_SetEtsTimeBuffers", "ps6000SetEtsTimeBuffers", c_uint32, [c_int16, c_void_p, c_void_p, c_uint32],
                   doc)

doc = """ PICO_STATUS ps6000RunBlock
    (
        int16_t           handle,
        uint32_t          noOfPreTriggerSamples,
        uint32_t          noOfPostTriggerSamples,
        uint32_t          timebase,
        int16_t           oversample,
        int32_t          *timeIndisposedMs,
        uint32_t          segmentIndex,
        ps6000BlockReady  lpReady,
        void             *pParameter
    ); """
ps6000.make_symbol("_RunBlock", "ps6000RunBlock", c_uint32,
                   [c_int16, c_uint32, c_uint32, c_uint32, c_int16, c_void_p, c_uint32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000IsReady
    (
        int16_t  handle,
        int16_t *ready
    ); """
ps6000.make_symbol("_IsReady", "ps6000IsReady", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps6000RunStreaming
    (
        int16_t            handle,
        uint32_t          *sampleInterval,
        PS6000_TIME_UNITS  sampleIntervalTimeUnits,
        uint32_t           maxPreTriggerSamples,
        uint32_t           maxPostPreTriggerSamples,
        int16_t            autoStop,
        uint32_t           downSampleRatio,
        PS6000_RATIO_MODE  downSampleRatioMode,
        uint32_t           overviewBufferSize
    ); """
ps6000.make_symbol("_RunStreaming", "ps6000RunStreaming", c_uint32,
                   [c_int16, c_void_p, c_int32, c_uint32, c_uint32, c_int16, c_uint32, c_int32, c_uint32], doc)
				   
doc = """ void ps6000StreamingReady
	(
		int16_t				handle,
		uint32_t			noOfSamples,
		uint32_t			startIndex,
		int16_t				overflow,
		uint32_t			triggerAt,
		int16_t				triggered,
		int16_t				autoStop,
		void				*pParameter
	); """
ps6000.StreamingReadyType = C_CALLBACK_FUNCTION_FACTORY(None,
														c_int16,
														c_uint32,
														c_uint32,
														c_int16,
														c_uint32,
														c_int16,
														c_int16,
														c_void_p)
														
ps6000.StreamingReadyType.__doc__ = doc

doc = """ PICO_STATUS ps6000GetStreamingLatestValues
    (
        int16_t               handle,
        ps6000StreamingReady  lpPs6000Ready,
        void                 *pParameter
    ); """
ps6000.make_symbol("_GetStreamingLatestValues", "ps6000GetStreamingLatestValues", c_uint32,
                   [c_int16, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000NoOfStreamingValues
    (
        int16_t   handle,
        uint32_t *noOfValues
    ); """
ps6000.make_symbol("_NoOfStreamingValues", "ps6000NoOfStreamingValues", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps6000GetMaxDownSampleRatio
    (
        int16_t            handle,
        uint32_t           noOfUnaggreatedSamples,
        uint32_t          *maxDownSampleRatio,
        PS6000_RATIO_MODE  downSampleRatioMode,
        uint32_t           segmentIndex
    ); """
ps6000.make_symbol("_GetMaxDownSampleRatio", "ps6000GetMaxDownSampleRatio", c_uint32,
                   [c_int16, c_uint32, c_void_p, c_int32, c_uint32], doc)

doc = """ PICO_STATUS ps6000GetValues
    (
        int16_t            handle,
        uint32_t           startIndex,
        uint32_t          *noOfSamples,
        uint32_t           downSampleRatio,
        PS6000_RATIO_MODE  downSampleRatioMode,
        uint32_t           segmentIndex,
        int16_t           *overflow
    ); """
ps6000.make_symbol("_GetValues", "ps6000GetValues", c_uint32,
                   [c_int16, c_uint32, c_void_p, c_uint32, c_int32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps6000GetValuesBulk
    (
        int16_t            handle,
        uint32_t          *noOfSamples,
        uint32_t           fromSegmentIndex,
        uint32_t           toSegmentIndex,
        uint32_t           downSampleRatio,
        PS6000_RATIO_MODE  downSampleRatioMode,
        int16_t           *overflow
    ); """
ps6000.make_symbol("_GetValuesBulk", "ps6000GetValuesBulk", c_uint32,
                   [c_int16, c_void_p, c_uint32, c_uint32, c_uint32, c_int32, c_void_p], doc)

doc = """ PICO_STATUS ps6000GetValuesAsync
    (
        int16_t            handle,
        uint32_t           startIndex,
        uint32_t           noOfSamples,
        uint32_t           downSampleRatio,
        PS6000_RATIO_MODE  downSampleRatioMode,
        uint32_t           segmentIndex,
        void              *lpDataReady,
        void              *pParameter
    ); """
ps6000.make_symbol("_GetValuesAsync", "ps6000GetValuesAsync", c_uint32,
                   [c_int16, c_uint32, c_uint32, c_uint32, c_int32, c_uint32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000GetValuesOverlapped
    (
        int16_t            handle,
        uint32_t           startIndex,
        uint32_t          *noOfSamples,
        uint32_t           downSampleRatio,
        PS6000_RATIO_MODE  downSampleRatioMode,
        uint32_t           segmentIndex,
        int16_t           *overflow
    ); """
ps6000.make_symbol("_GetValuesOverlapped", "ps6000GetValuesOverlapped", c_uint32,
                   [c_int16, c_uint32, c_void_p, c_uint32, c_int32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps6000GetValuesOverlappedBulk
    (
        int16_t            handle,
        uint32_t           startIndex,
        uint32_t          *noOfSamples,
        uint32_t           downSampleRatio,
        PS6000_RATIO_MODE  downSampleRatioMode,
        uint32_t           fromSegmentIndex,
        uint32_t           toSegmentIndex,
        int16_t           *overflow
    ); """
ps6000.make_symbol("_GetValuesOverlappedBulk", "ps6000GetValuesOverlappedBulk", c_uint32,
                   [c_int16, c_uint32, c_void_p, c_uint32, c_int32, c_uint32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps6000GetValuesBulkAsyc
    (
        int16_t            handle,
        uint32_t           startIndex,
        uint32_t          *noOfSamples,
        uint32_t           downSampleRatio,
        PS6000_RATIO_MODE  downSampleRatioMode,
        uint32_t           fromSegmentIndex,
        uint32_t           toSegmentIndex,
        int16_t           *overflow
    ); """
ps6000.make_symbol("_GetValuesAsync", "ps6000GetValuesAsync", c_uint32,
                   [c_int16, c_uint32, c_uint32, c_uint32, c_int32, c_uint32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps6000GetNoOfCaptures
    (
        int16_t   handle,
        uint32_t *nCaptures
    ); """
ps6000.make_symbol("_GetNoOfCaptures", "ps6000GetNoOfCaptures", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps6000GetNoOfProcessedCaptures
    (
        int16_t   handle,
        uint32_t *nProcessedCaptures
    ); """
ps6000.make_symbol("_GetNoOfProcessedCaptures", "ps6000GetNoOfProcessedCaptures", c_uint32, [c_int16, c_void_p], doc)

""" PICO_STATUS ps6000Stop
    (
        int16_t  handle
    ); """
ps6000.make_symbol("_Stop", "ps6000Stop", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps6000SetNoOfCaptures
    (
        int16_t   handle,
        uint32_t  nCaptures
    ); """
ps6000.make_symbol("_SetNoOfCaptures", "ps6000SetNoOfCaptures", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps6000SetWaveformLimiter
    (
        int16_t   handle,
        uint32_t  nWaveformsPerSecond
    ); """
ps6000.make_symbol("_SetWaveformLimiter", "ps6000SetWaveformLimiter", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps6000EnumerateUnits
    (
        int16_t *count,
        int8_t  *serials,
        int16_t *serialLth
    ); """
ps6000.make_symbol("_EnumerateUnits", "ps6000EnumerateUnits", c_uint32, [c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000SetExternalClock
    (
        int16_t                    handle,
        PS6000_EXTERNAL_FREQUENCY  frequency,
        int16_t                    threshold
    ); """
ps6000.make_symbol("_SetExternalClock", "ps6000SetExternalClock", c_uint32, [c_int16, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps6000PingUnit
    (
        int16_t  handle
    ); """
ps6000.make_symbol("_PingUnit", "ps6000PingUnit", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps6000GetAnalogueOffset
    (
        int16_t          handle,
        PS6000_RANGE     range,
        PS6000_COUPLING  coupling,
        float           *maximumVoltage,
        float           *minimumVoltage
    ); """
ps6000.make_symbol("_GetAnalogueOffset", "ps6000GetAnalogueOffset", c_uint32,
                   [c_int16, c_int32, c_int32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps6000GetTriggerInfoBulk
    (
        int16_t        handle,
        PS6000_TRIGGER_INFO    *triggerInfo,
        uint32_t      fromSegmentIndex,
        uint32_t      toSegmentIndex
    ); """
ps6000.make_symbol("_GetTriggerInfoBulk", "ps6000GetTriggerInfoBulk", c_uint16, [c_int16, c_void_p, c_uint32, c_uint32], doc)



#
# Copyright (C) 2015-2018 Pico Technology Ltd. See LICENSE file for terms.
#
"""
This is a Python module defining the functions from the usbtc08.h C header file
for TC-08 Thermocouple Data Logger using the usbtc08 driver API functions.
"""

from ctypes import *
from picosdk.library import Library
from picosdk.errors import ArgumentOutOfRangeError
from picosdk.constants import make_enum

class usbtc08lib(Library):
    def __init__(self):
        super(usbtc08lib, self).__init__("usbtc08")


usbtc08 = usbtc08lib()

usbtc08.USBTC08_UNITS = make_enum([
    "USBTC08_UNITS_CENTIGRADE",
	"USBTC08_UNITS_FAHRENHEIT",
	"USBTC08_UNITS_KELVIN",
	"USBTC08_UNITS_RANKINE",
])

class USBTC08_INFO(Structure):
    _pack_ = 1
    _fields_ = [("size", c_int16),
                ("DriverVersion", c_char * 12),
                ("PicoppVersion", c_int16),
				("HardwareVersion", c_int16),
				("Variant", c_int16),
				("szSerial[USBTC08_MAX_SERIAL_CHAR]", c_char * 11),
				("szCalDate[USBTC08_MAX_DATE_CHARS]", c_char * 9)]

doc = """ int16_t usb_tc08_open_unit
    (
	    void
	); """
usbtc08.make_symbol("_open_unit_","usb_tc08_open_unit", c_int16, [], doc)

doc = """ int16_t usb_tc08_open_unit_async
    (
	    void
	); """
usbtc08.make_symbol("_open_unit_async_","usb_tc08_open_unit_async", c_int16, [], doc)

doc = """ int16_t usb_tc08_open_unit_progress
    (
	    int16_t  *handle,
		int16_t  *progress
	); """
usbtc08.make_symbol("_open_unit_progress_","usb_tc08_open_unit_progress", c_int16, [c_void_p, c_void_p], doc)

doc = """ int16_t usb_tc08_close_unit
    (
	    int16_t  handle
	); """
usbtc08.make_symbol("_close_unit_","usb_tc08_close_unit", c_int16, [c_int16], doc)

doc = """ int16_t usb_tc08_stop
    (
	    int16_t  handle
	); """
usbtc08.make_symbol("_stop_","usb_tc08_stop", c_int16, [c_int16], doc)

doc = """ int16 usb_tc08_set_mains
    (
	    int16_t  handle,
		int16_t  sixty_hertz
	); """
usbtc08.make_symbol("_set_mains_","usb_tc08_set_mains", c_int16, [c_int16, c_int16], doc)

doc = """ int32_t usb_tc08_get_minimum_interval_ms
    (
	    int16_t  handle
	); """
usbtc08.make_symbol("_get_minimum_interval_ms_","usb_tc08_get_minimum_interval_ms", c_int16, [c_int16], doc)

doc = """ int16_t usb_tc08_get_unit_info
    (
	    int16_t  handle
		USBTC08_INFO  *info
	); """
usbtc08.make_symbol("_get_unit_info_","usb_tc08_get_unit_info", c_int16, [c_int16, c_void_p], doc)

doc = """ int16_t usb_tc08_get_unit_info2
    (
	    int16_t  handle,
		int8_t  *string,
		int16_t  string_length,
		int16_t  line
	); """
usbtc08.make_symbol("_get_unit_info2_","usb_tc08_get_unit_info2", c_int16, [c_int16, c_void_p, c_int16, c_int16], doc)

doc = """ int16_t usb_tc08_get_formatted_info
    (
	    int16_t  handle,
		int8_t  *unit_info,
		int16_t  string_length
	); """
usbtc08.make_symbol("_get_formatted_info_","usb_tc08_get_formatted_info", c_int16, [c_int16, c_void_p, c_int16], doc)

doc = """ int16_t usb_tc08_get_last_error
    (
	    int16_t  handle
	); """
usbtc08.make_symbol("_get_last_error_","usb_tc08_get_last_error", c_int16, [c_int16], doc)

doc = """ int16_t usb_tc08_set_channel
    (
	    int16_t  handle,
		int16_t  channel,
		int8_t  tc_type
	); """
usbtc08.make_symbol("_set_channel_","usb_tc08_set_channel", c_int16, [c_int16, c_int16, c_int8], doc)

doc = """ int32_t usb_tc08_run
    (
	    int16_t  handle,
		int32_t  interval
	); """
usbtc08.make_symbol("_run_","usb_tc08_run", c_int16, [c_int16, c_int32], doc)

doc = """ int16_t usb_tc08_get_single
    (
	    int16_t  handle,
		float  *temp,
		int16_t  *overflow_flags,
		int16_t  units
	); """
usbtc08.make_symbol("_get_single_","usb_tc08_get_single", c_int16, [c_int16, c_void_p, c_void_p, c_int16], doc)

doc = """ int32_t usb_tc08_get_temp
    (
	    int16_t  handle,
		float  *temp_buffer,
		int32_t  *times_ms_buffer,
		int32_t  buffer_length,
		int16_t  *overflow,
		int16_t  channel,
		int16_t  units,
		int16_t  fill_missing
	); """
usbtc08.make_symbol("_get_temp_","usb_tc08_get_temp", c_int16, [c_int16, c_void_p, c_void_p, c_int32, c_void_p, c_void_p, c_int16, c_int16], doc)

doc = """ int32_t usb_tc08_get_temp_deskew
    (
	    int16_t  handle,
		float  *temp_buffer,
		int32_t  *times,
		int32_t  buffer_length,
		int16_t  *overflow,
		int16_t  channel,
		int16_t  units,
		int16_t  fill_missing
	); """
usbtc08.make_symbol("_get_temp_deskew_","usb_tc08_get_temp_deskew", c_int16, [c_int16, c_void_p, c_void_p, c_int32, c_void_p, c_int16, c_int16, c_int16], doc)


#
# Copyright (C) 2019 Pico Technology Ltd. See LICENSE file for terms.
#
"""
This is a Python module defining the functions from the ps5000Api.h C header
file for PicoScope 5000 Series oscilloscopes using the ps5000 driver API
functions.
"""

from ctypes import *
from picosdk.ctypes_wrapper import C_CALLBACK_FUNCTION_FACTORY
from picosdk.library import Library
from picosdk.constants import make_enum

class Ps5000lib(Library):
    def __init__(self):
        super(Ps5000lib, self).__init__("ps5000")


ps5000 = Ps5000lib()

ps5000.PS5000_CHANNEL = make_enum([
	"PS5000_CHANNEL_A",
	"PS5000_CHANNEL_B",
	"PS5000_CHANNEL_C",
	"PS5000_CHANNEL_D",
	("PS5000_MAX_CHANNELS", "PS5000_EXTERNAL"),
	"PS5000_TRIGGER_AUX",
	"PS5000_MAX_TRIGGER_SOURCES",
])

ps5000.PS5000_RANGE = make_enum([
	"PS5000_10MV",
	"PS5000_20MV",
	"PS5000_50MV",
	"PS5000_100MV",
	"PS5000_200MV",
    "PS5000_500MV",
	"PS5000_1V",
	"PS5000_2V",
	"PS5000_5V",
	"PS5000_10V",
	"PS5000_20V",
	"PS5000_50V",
	"PS5000_MAX_RANGES",
])

ps5000.PS5000_TIME_UNITS = make_enum([
	"PS5000_FS",
	"PS5000_PS",
	"PS5000_NS",
	"PS5000_US",
	"PS5000_MS",
	"PS5000_S",
	"PS5000_MAX_TIME_UNITS",
])
	

class PWQ_CONDITIONS (Structure):
	_pack = 1
	_fields = [("channelA", c_int32),
				("channelB", c_int32),
				("channelC", c_int32),
				("channelD", c_int32),
				("external", c_int32),
				("aux", c_int32)]

ps5000.PWQ_CONDITIONS = PWQ_CONDITIONS

class TRIGGER_CONDITIONS (Structure):
	_pack = 1
	_fields = [("channelA", c_int32),
				("channelB", c_int32),
				("channelC", c_int32),
				("channelD", c_int32),
				("external", c_int32),
				("aux", c_int32),
				("pulseWidthQualifier", c_int32)]
				
ps5000.TRIGGER_CONDITIONS = TRIGGER_CONDITIONS

class TRIGGER_CHANNEL_PROPERTIES (Structure):
	_pack = 1
	_fields = [("thresholdMajor", c_int16),
				("thresholdMinor", c_int16),
				("hysteresis", c_uint16),
				("channel", c_int32),
				("thresholdMode", c_int32)]
				
ps5000.TRIGGER_CHANNEL_PROPERTIES = TRIGGER_CHANNEL_PROPERTIES
				
doc = """ PICO_STATUS ps5000CloseUnit
    (
        short  handle
    ); """
ps5000.make_symbol("_CloseUnit", "ps5000CloseUnit", c_uint32, [c_int16], doc)

doc = """ PICO_STATUS ps5000FlashLed
    (
        short  handle,
        short  start
    ); """
ps5000.make_symbol("_FlashLed", "ps5000FlashLed", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps5000GetMaxDownSampleRatio
    (
        short  handle,
        unsigned long  noOfUnaggregatedSamples,
        unsigned long  *maxDownSampleRatio,
        short  downSampleRatioMode,
        unsigned short  segmentIndex
    ); """
ps5000.make_symbol("_GetMaxDownSampleRatio", "ps5000GetMaxDownSampleRatio", c_uint32, [c_int16, c_uint32, c_void_p, c_int16, c_uint16], doc)

doc = """ PICO_STATUS ps5000GetStreamingLatestValues
    (
        short  handle,
        ps5000StreamingReady  lpPs5000Ready,
        void  *pParameter
    ); """
ps5000.make_symbol("_GetStreamingLatestValues", "ps5000GetStreamingLatestValues", c_uint16, [c_int16, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps5000GetTimebase
    (
        short  handle,
        unsigned long  timebase,
        long  noSamples,
        long  *timeIntervalNanoseconds,
        short  oversample,
        long  *maxSamples,
        unsigned short  segmentIndex
    ); """
ps5000.make_symbol("_GetTimebase", "ps5000GetTimebase", c_uint16, [c_int16, c_uint32, c_int32, c_void_p, c_int16, c_void_p, c_uint16], doc)

doc = """ PICO_STATUS ps5000GetTriggerTimeOffset
    (
        short  handle,
        unsigned long  *timeUpper,
        unsigned long  *timeLower,
        PS5000_TIME_UNITS  *timeUnits,
        unsigned short  segmentIndex
    ); """
ps5000.make_symbol("_GetTriggerTimeOffset", "ps5000GetTriggerTimeOffset", c_uint16, [c_int16, c_void_p, c_void_p, c_void_p, c_uint16], doc)

doc = """ PICO_STATUS ps5000GetTriggerTimeOffset64
    (
        short  handle,
        int64_t  *time,
        PS5000_TIME_UNITS  *timeUnits,
        unsigned short  segmentIndex
    ); """
ps5000.make_symbol("_GetTriggerTimeOffset64", "ps5000GetTriggerTimeOffset64", c_uint32, [c_int16, c_void_p, c_void_p, c_uint16], doc)

doc = """ PICO_STATUS ps5000GetUnitInfo
    (
        short  handle,
        char  *string,
        short  stringLength,
        short  *requiredSize,
        PICO_INFO  info
    ); """
ps5000.make_symbol("_GetUnitInfo", "ps5000GetUnitInfo", c_uint32, [c_int16, c_void_p, c_int16, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps5000GetValues
    (
        short  handle,
        unsigned long  startIndex,
        unsigned long  *noOfSamples,
        unsigned long  downSampleRatio,
        short  downSampleRatioMode,
        unsigned short  segmentIndex,
        short  *overflow
    ); """
ps5000.make_symbol("_GetValues", "ps5000GetValues", c_uint32, [c_int16, c_uint32, c_void_p, c_uint32, c_int16, c_uint16, c_void_p], doc)

doc = """ PICO_STATUS ps5000GetValuesAsync
    (
        short  handle,
        unsigned long  startIndex,
        unsigned long  noOfSamples,
        unsigned long  downSampleRatio,
        short  downSampleRatioMode,
        unsigned short  segmentIndex,
        void  *lpDataReady,
        void  *pParameter
    ); """
ps5000.make_symbol("_GetValuesAsync", "ps5000GetValuesAsync", c_uint32, [c_int16, c_uint32, c_uint32, c_uint32, c_int16, c_uint16, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps5000GetValuesBulk
    (
        short  handle,
        unsigned long  *noOfSamples,
        unsigned short  fromSegmentIndex,
        unsigned short  toSegmentIndex,
        short  *overflow
    ); """
ps5000.make_symbol("_GetValuesBulk", "ps5000GetValuesBulk", c_uint32, [c_int16, c_void_p, c_uint16, c_uint16, c_void_p], doc)

doc = """ PICO_STATUS ps5000GetValuesTriggerTimeOffsetBulk
	(
		short  handle,
		unsigned long  *timesUpper,
		unsigned long  *timesLower,
		PS5000_TIME_UNITS  *timeUnits,
		unsigned short   fromSegmentIndex,
		unsigned short  toSegmentIndex
	); """
ps5000.make_symbol("_GetValuesTriggerTimeOffsetBulk", "ps5000GetValuesTriggerTimeOffsetBulk", c_uint32, [c_int16, c_void_p, c_void_p, c_void_p, c_uint16, c_uint16], doc)

doc = """ PICO_STATUS ps5000GetValuesTriggerTimeOffsetBulk64
	(
		short  handle,
		_int64  *times,
		PS5000_TIME_UNITS  *timeUnits,
		unsigned short  fromSegmentIndex,
		unsigned short  toSegmentIndex
	); """
ps5000.make_symbol("_GetValuesTriggerTimeOffsetBulk64", "ps5000GetValuesTriggerTimeOffsetBulk64", c_uint32, [c_int16, c_void_p, c_void_p, c_uint16, c_int16], doc)

doc = """ PICO_STATUS ps5000IsLedFlashing
	(
		short  handle,
		short  *status
	); """
ps5000.make_symbol("_IsLedFlashing", "ps5000IsLedFlashing", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps5000IsReady
	(
		short  handle,
		short  *ready
	); """
ps5000.make_symbol("_IsReady", "ps5000IsReady", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps5000IsTriggerOrPulseWidthQualifierEnabled
	(
		short  handle,
		short  *triggerEnabled,
		short  *pulseWidthQualifierEnabled
	); """
ps5000.make_symbol("_IsTriggerOrPulseWidthQualifierEnabled", "ps5000IsTriggerOrPulseWidthQualifierEnabled", c_uint32, [c_int16, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps5000MemorySegments
	(
		short  handle,
		unsigned short  nSegments,
		long  *nMaxSamples
	); """
ps5000.make_symbol("_MemorySegments", "ps5000MemorySegments", c_uint32, [c_int16, c_uint16, c_void_p], doc)

doc = """ PICO_STATUS ps5000NoOfStreamingValues
	(
		short  handle,
		unsigned long  *noOfValues
	); """
ps5000.make_symbol("_NoOfStreamingValues", "ps5000NoOfStreamingValues", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps5000OpenUnit
	(
		short  *handle
	); """
ps5000.make_symbol("_OpenUnit", "ps5000OpenUnit", c_uint32, [c_void_p], doc)

doc = """ PICO_STATUS ps5000OpenUnitAsync
	(
		short  *status
	); """
ps5000.make_symbol("_OpenUnitAsync", "ps5000OpenUnitAsync", c_uint32, [c_void_p], doc)

doc = """ PICO_STATUS ps5000OpenUnitProgress
	(
		short  *handle,
		short  *progressPercent,
		short  *complete
	); """
ps5000.make_symbol("_OpenUnitProgress", "ps5000OpenUnitProgress", c_uint32, [c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps5000RunBlock
	(
		short  handle,
		long  noOfPreTriggerSamples,
		long  noOfPostTriggerSamples,
		unsigned long  timebase,
		short  oversample,
		long  *timeIndisposedMs,
		unsigned short  segmentIndex,
		ps5000BlockReady  lpReady,
		void  *pParameter
	); """
ps5000.make_symbol("_RunBlock", "ps5000RunBlock", c_uint32, [c_int16, c_int32, c_int32, c_uint32, c_int16, c_void_p, c_uint16, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps5000RunStreaming
	(
		short  handle,
		unsigned long  *sampleInterval,
		PS5000_TIME_UNITS  sampleIntervalTimeUnits,
		unsigned long  maxPreTriggerSamples,
		unsigned long  maxPostTriggerSamples,
		short  autoStop,
		unsigned long  downSampleRatio,
		unsigned long  overviewBufferSize
	); """
ps5000.make_symbol("_RunStreaming", "ps5000RunStreaming", c_uint32, [c_int16, c_void_p, c_int32, c_uint32, c_uint32, c_int16, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS ps5000SetChannel
	(
		short  handle,
		PS5000_CHANNEL  channel,
		short  enabled,
		short  dc,
		PS5000_RANGE  range
	); """
ps5000.make_symbol("_SetChannel", "ps5000SetChannel", c_uint32, [c_int16, c_int32, c_int16, c_int16, c_int32], doc)

doc = """ PICO_STATUS ps5000SetDataBuffer
	(
		short  handle,
		PS5000_CHANNEL  channel,
		short  *buffer,
		long  buggerLth
	); """
ps5000.make_symbol("_SetDataBuffer", "ps5000SetDataBuffer", c_uint32, [c_int16, c_int32, c_void_p, c_int32], doc)

doc = """ PICO_STATUS ps5000SetDataBufferBulk
	(
		short  handle,
		PS5000_CHANNEL  channel,
		short  *buffer,
		long  bufferLth,
		unsigned short  waveform
	); """
ps5000.make_symbol("_SetDataBufferBulk", "ps5000SetDataBufferBulk", c_uint32, [c_int16, c_int32, c_void_p, c_int32, c_uint16], doc)

doc = """ PICO_STATUS ps5000SetDataBuffers
	(
		short  handle,
		PS5000_CHANNEL  channel,
		short  *bufferMax,
		short  *bufferMin,
		long  bufferLth
	); """
ps5000.make_symbol("_SetDataBuffers", "ps5000SetDataBuffers", c_uint32, [c_int16, c_int32, c_void_p, c_void_p, c_int32], doc)

doc = """ PICO_STATUS ps5000SetEts
	(
		short  handle,
		PS5000_ETS_MODE  mode,
		short  etsCycles,
		short  etsInterleave,
		long  *sampleTimePicoseconds
	); """
ps5000.make_symbol("_SetEts", "ps5000SetEts", c_uint32, [c_int16, c_int32, c_int16, c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps5000SetEtsTimeBuffer
	(
		short  handle,
		_int64  *buffer,
		long  bufferLth
	); """
ps5000.make_symbol("_SetEtsTimeBuffer", "ps5000SetEtsTimeBuffer", c_uint32, [c_int16, c_void_p, c_int32], doc)

doc = """ PICO_STATUS ps5000SetEtsTimeBuffers
	(
		short  handle,
		unsigned long  *timeUpper,
		unsigned long  *timeLower,
		long  bufferLth
	); """
ps5000.make_symbol("_SetEtsTimeBuffers", "ps5000SetEtsTimeBuffers", c_uint32, [c_int16, c_void_p, c_void_p, c_int32], doc)

doc = """ PICO_STATUS ps5000SetNoOfCaptures
	(
		short  handle,
		unsigned short  nCaptures
	); """
ps5000.make_symbol("_SetNoOfCaptures", "ps5000SetNoOfCaptures", c_uint32, [c_int16, c_uint16], doc)

doc = """ PICO_STATUS ps5000SetPulseWidthQualifier
	(
		short  handle,
		struct PWQ_CONDITIONS  *conditions,
		short  nConditions,
		THRESHOLD_DIRECTION  direction,
		unsigned long  lower,
		unsigned long  upper,
		PULSE_WIDTH_TYPE  type
	); """
ps5000.make_symbol("_SetPulseWidthQualifier", "ps5000SetPulseWidthQualifier", c_uint32, [c_int16, c_void_p, c_int16, c_int32, c_uint32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps5000SetSigGenArbitrary
	(
		short  handle,
		long  offsetVoltage,
		unsigned long  pkToPk,
		unsigned long  startDeltaPhase,
		unsigned long  stopDeltaPhase,
		unsigned long  deltaPhaseIncrement,
		unsigned long  dwellCount,
		short  *arbitaryWaveform
		long  arbitaryWaveformSize,
		SWEEP_TYPE  sweepType,
		short  whiteNoise,
		INDEX_MODE  indexMode,
		unsigned long  shots,
		unsigned long sweeps,
		SIGGEN_TRIG_TYPE  triggerType,
		SIGGEN_TRIG_SOURCE  triggerSource,
		short  extInThreshold
	); """
ps5000.make_symbol("_SetSigGenArbitrary", "ps5000SetSigGenArbitrary", c_uint32, 
					[c_int16, c_int32, c_uint32, c_uint32, c_uint32, c_uint32, c_uint32, c_void_p, 
						c_int32, c_int32, c_int16, c_int32, c_uint32, c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps5000SetSigGenBuiltIn
	(
		short  handle,
		long  offsetVoltage,
		unsigned long  pkToPk,
		short  waveType,
		float  startFrequency,
		float  stopFrequency,
		float  increment,
		float  dwellTime,
		SWEEP_TYPE  sweepType,
		short  whiteNoise,
		unsigned long  shots,
		unsigned long  sweeps,
		SIGGEN_TRIG_TYPE  triggerType,
		SIGGEN_TRIG_SOURCE  triggerSource,
		short  extInThreshold
	); """
ps5000.make_symbol("_SetSigGenBuiltIn", "ps5000SetSigGenBuiltIn", c_uint32, 
					[c_int16, c_int32, c_uint32, c_int16, c_int64, c_int64, c_int64, c_int64, c_int64, c_int32, c_int16, c_uint32, c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps5000SetSimpleTrigger
	(
		short  handle,
		short  enable,
		PS5000_CHANNEL  source,
		short  threshold,
		THRESHOLD_DIRECTION  direction,
		unsigned long  delay,
		short  autoTrigger_ms
	); """
ps5000.make_symbol("_SetSimpleTrigger", "ps5000SetSimpleTrigger", c_uint32, [c_int16, c_int16, c_int32, c_int16, c_int32, c_uint32, c_int16], doc)

doc = """ PICO_STATUS ps5000SetTriggerChannelConditions
	(
		short  handle,
		struct TRIGGER_CONDITIONS  *conditions,
		short  nConditions
	); """
ps5000.make_symbol("_SetTriggerChannelConditions", "ps5000SetTriggerChannelConditions", c_uint32, [c_int16, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps5000SetTriggerChannelDirections
	(
		short  handle,
		THRESHOLD_DIRECTION  channelA,
		THRESHOLD_DIRECTION  channelB,
		THRESHOLD_DIRECTION  channelC,
		THRESHOLD_DIRECTION  channelD,
		THRESHOLD_DIRECTION  ext,
		THRESHOLD_DIRECTION  aux
	); """
ps5000.make_symbol("_SetTriggerChannelDirections", "ps5000SetTriggerChannelDirections", c_uint32, [c_int16, c_int32, c_int32, c_int32, c_int32, c_int32, c_int32], doc)

doc = """ PICO_STATUS ps5000SetTriggerChannelProperties
	(
		short  handle,
		struct TRIGGER_CHANNEL_PROPERTIES  *channelProperties,
		short  nChannelProperties,
		short  auxOutputEnable,
		long  autoTriggerMilliseconds
	); """
ps5000.make_symbol("_SetTriggerChannelProperties", "ps5000SetTriggerChannelProperties", c_uint32, [c_int16, c_void_p, c_int16, c_int16, c_int32], doc)

doc = """ PICO_STATUS ps5000SetTriggerDelay
	(
		short  handle,
		unsigned long  delay
	); """
ps5000.make_symbol("_SetTriggerDelay", "ps5000SetTriggerDelay", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps5000SigGenSoftwareControl
	(
		short  handle,
		short  state
	); """
ps5000.make_symbol("_SigGenSoftwareControl", "ps5000SigGenSoftwareControl", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps5000Stop
	(
		short  handle
	); """
ps5000.make_symbol("_Stop", "ps5000Stop", c_uint32, [c_int16], doc)

doc = """ void ps5000BlockReady
	(
		short  handle,
		PICO_STATUS  status,
		void  *pParameter
	); """

ps5000.BlockReadyType = C_CALLBACK_FUNCTION_FACTORY(None,
													c_int16,
													c_uint32,
													c_void_p)
													
ps5000.BlockReadyType.__doc__ = doc

doc = """ void (CALLBACK *ps5000DataReady)
	(
		short  handle,
		long  noOfSamples,
		short  overflow,
		unsigned long  triggerAt,
		short  triggered,
		void  *pParameter
	); """
	
ps5000.DataReadyType = C_CALLBACK_FUNCTION_FACTORY(None,
													c_int16,
													c_int32,
													c_int16,
													c_uint32,
													c_int16,
													c_void_p)
													
ps5000.DataReadyType.__doc__ = doc

doc = """ void (CALLBACK *ps5000StreamingReady)
	(
		short  handle,
		long  noOfSamples,
		unsigned long  startIndex,
		short  overflow,
		unsigned long  triggerAt,
		short  triggered,
		short  autoStop,
		void  *pParameter
	); """

ps5000.StreamingReadyType = C_CALLBACK_FUNCTION_FACTORY(None,
														c_int16,
														c_int32,
														c_uint32, 
														c_int16,
														c_uint32,
														c_int16,
														c_int16,
														c_void_p)
														
ps5000.StreamingReadyType.__doc__ = doc


#
# Copyright (C) 2020 Pico Technology Ltd. See LICENSE file for terms.
#
# PS6000 A BLOCK MODE EXAMPLE
# This example opens a 6000a driver device, sets up two channels and a trigger then collects 10 blocks of data in rapid block mode.
# This data is then plotted as mV against time in ns.

import ctypes
import numpy as np
from picosdk.ps6000a import ps6000a as ps
from picosdk.PicoDeviceEnums import picoEnum as enums
import matplotlib.pyplot as plt
from picosdk.functions import adc2mV, assert_pico_ok

# Create chandle and status ready for use
chandle = ctypes.c_int16()
status = {}

# Open 6000 A series PicoScope
# returns handle to chandle for use in future API functions
resolution = enums.PICO_DEVICE_RESOLUTION["PICO_DR_8BIT"]
status["openunit"] = ps.ps6000aOpenUnit(ctypes.byref(chandle), None, resolution)
assert_pico_ok(status["openunit"])

# Set channel A on
# handle = chandle
channelA = enums.PICO_CHANNEL["PICO_CHANNEL_A"]
coupling = enums.PICO_COUPLING["PICO_DC"]
channelRange = 7
# analogueOffset = 0 V
bandwidth = enums.PICO_BANDWIDTH_LIMITER["PICO_BW_FULL"]
status["setChannelA"] = ps.ps6000aSetChannelOn(chandle, channelA, coupling, channelRange, 0, bandwidth)
assert_pico_ok(status["setChannelA"])

# set channel B-H off
for x in range (1, 7, 1):
    channel = x
    status["setChannel",x] = ps.ps6000aSetChannelOff(chandle,channel)
    assert_pico_ok(status["setChannel",x])

# Set simple trigger on channel A, 1 V rising with 1 s autotrigger
# handle = chandle
# enable = 1
source = channelA
# threshold = 1000 mV
direction = enums.PICO_THRESHOLD_DIRECTION["PICO_RISING"]
# delay = 0 s
# autoTriggerMicroSeconds = 1000000 us
status["setSimpleTrigger"] = ps.ps6000aSetSimpleTrigger(chandle, 1, source, 1000, direction, 0, 1000000)
assert_pico_ok(status["setSimpleTrigger"])

# Get fastest available timebase
# handle = chandle
enabledChannelFlags = enums.PICO_CHANNEL_FLAGS["PICO_CHANNEL_A_FLAGS"]
timebase = ctypes.c_uint32(0)
timeInterval = ctypes.c_double(0)
# resolution = resolution
status["getMinimumTimebaseStateless"] = ps.ps6000aGetMinimumTimebaseStateless(chandle, enabledChannelFlags, ctypes.byref(timebase), ctypes.byref(timeInterval), resolution)
assert_pico_ok(status["getMinimumTimebaseStateless"])
print("timebase = ", timebase.value)
print("sample interval =", timeInterval.value, "s")

# Set number of samples to be collected
noOfPreTriggerSamples = 500000
noOfPostTriggerSamples = 1000000
nSamples = noOfPostTriggerSamples + noOfPreTriggerSamples

# Set number of memory segments
noOfCaptures = 10
maxSegments = ctypes.c_uint64(10)
status["memorySegments"] = ps.ps6000aMemorySegments(chandle, noOfCaptures, ctypes.byref(maxSegments))
assert_pico_ok(status["memorySegments"])

# Set number of captures
status["setNoOfCaptures"] = ps.ps6000aSetNoOfCaptures(chandle, noOfCaptures)
assert_pico_ok(status["setNoOfCaptures"])

# Create buffers
bufferAMax = (ctypes.c_int16 * nSamples)()
bufferAMin = (ctypes.c_int16 * nSamples)() # used for downsampling which isn't in the scope of this example
bufferAMax1 = (ctypes.c_int16 * nSamples)()
bufferAMin1 = (ctypes.c_int16 * nSamples)() # used for downsampling which isn't in the scope of this example
bufferAMax2 = (ctypes.c_int16 * nSamples)()
bufferAMin2 = (ctypes.c_int16 * nSamples)() # used for downsampling which isn't in the scope of this example
bufferAMax3 = (ctypes.c_int16 * nSamples)()
bufferAMin3 = (ctypes.c_int16 * nSamples)() # used for downsampling which isn't in the scope of this example
bufferAMax4 = (ctypes.c_int16 * nSamples)()
bufferAMin4 = (ctypes.c_int16 * nSamples)() # used for downsampling which isn't in the scope of this example
bufferAMax5 = (ctypes.c_int16 * nSamples)()
bufferAMin5 = (ctypes.c_int16 * nSamples)() # used for downsampling which isn't in the scope of this example
bufferAMax6 = (ctypes.c_int16 * nSamples)()
bufferAMin6 = (ctypes.c_int16 * nSamples)() # used for downsampling which isn't in the scope of this example
bufferAMax7 = (ctypes.c_int16 * nSamples)()
bufferAMin7 = (ctypes.c_int16 * nSamples)() # used for downsampling which isn't in the scope of this example
bufferAMax8 = (ctypes.c_int16 * nSamples)()
bufferAMin8 = (ctypes.c_int16 * nSamples)() # used for downsampling which isn't in the scope of this example
bufferAMax9 = (ctypes.c_int16 * nSamples)()
bufferAMin9 = (ctypes.c_int16 * nSamples)() # used for downsampling which isn't in the scope of this example

# Set data buffers
# handle = chandle
# channel = channelA
# bufferMax = bufferAMax
# bufferMin = bufferAMin
# nSamples = nSamples
dataType = enums.PICO_DATA_TYPE["PICO_INT16_T"]
waveform = 0
downSampleMode = enums.PICO_RATIO_MODE["PICO_RATIO_MODE_RAW"]
clear = enums.PICO_ACTION["PICO_CLEAR_ALL"]
add = enums.PICO_ACTION["PICO_ADD"]
action = clear|add # PICO_ACTION["PICO_CLEAR_WAVEFORM_CLEAR_ALL"] | PICO_ACTION["PICO_ADD"]  
status["setDataBuffers"] = ps.ps6000aSetDataBuffers(chandle, channelA, ctypes.byref(bufferAMax), ctypes.byref(bufferAMin), nSamples, dataType, waveform, downSampleMode, action)
assert_pico_ok(status["setDataBuffers"])
waveform1 = 1
status["setDataBuffers1"] = ps.ps6000aSetDataBuffers(chandle, channelA, ctypes.byref(bufferAMax1), ctypes.byref(bufferAMin1), nSamples, dataType, waveform1, downSampleMode, add)
assert_pico_ok(status["setDataBuffers1"])
waveform2 = 2
status["setDataBuffers2"] = ps.ps6000aSetDataBuffers(chandle, channelA, ctypes.byref(bufferAMax2), ctypes.byref(bufferAMin2), nSamples, dataType, waveform2, downSampleMode, add)
assert_pico_ok(status["setDataBuffers2"])
waveform3 = 3
status["setDataBuffers3"] = ps.ps6000aSetDataBuffers(chandle, channelA, ctypes.byref(bufferAMax3), ctypes.byref(bufferAMin3), nSamples, dataType, waveform3, downSampleMode, add)
assert_pico_ok(status["setDataBuffers3"])
waveform4 = 4
status["setDataBuffers4"] = ps.ps6000aSetDataBuffers(chandle, channelA, ctypes.byref(bufferAMax4), ctypes.byref(bufferAMin4), nSamples, dataType, waveform4, downSampleMode, add)
assert_pico_ok(status["setDataBuffers4"])
waveform5 = 5
status["setDataBuffers5"] = ps.ps6000aSetDataBuffers(chandle, channelA, ctypes.byref(bufferAMax5), ctypes.byref(bufferAMin5), nSamples, dataType, waveform5, downSampleMode, add)
assert_pico_ok(status["setDataBuffers5"])
waveform6 = 6
status["setDataBuffers6"] = ps.ps6000aSetDataBuffers(chandle, channelA, ctypes.byref(bufferAMax6), ctypes.byref(bufferAMin6), nSamples, dataType, waveform6, downSampleMode, add)
assert_pico_ok(status["setDataBuffers6"])
waveform7 = 7
status["setDataBuffers7"] = ps.ps6000aSetDataBuffers(chandle, channelA, ctypes.byref(bufferAMax7), ctypes.byref(bufferAMin7), nSamples, dataType, waveform7, downSampleMode, add)
assert_pico_ok(status["setDataBuffers7"])
waveform8 = 8
status["setDataBuffers8"] = ps.ps6000aSetDataBuffers(chandle, channelA, ctypes.byref(bufferAMax8), ctypes.byref(bufferAMin8), nSamples, dataType, waveform8, downSampleMode, add)
assert_pico_ok(status["setDataBuffers8"])
waveform9 = 9
status["setDataBuffers9"] = ps.ps6000aSetDataBuffers(chandle, channelA, ctypes.byref(bufferAMax9), ctypes.byref(bufferAMin9), nSamples, dataType, waveform9, downSampleMode, add)
assert_pico_ok(status["setDataBuffers9"])

# Run block capture
# handle = chandle
# timebase = timebase
timeIndisposedMs = ctypes.c_double(0)
# segmentIndex = 0
# lpReady = None   Using IsReady rather than a callback
# pParameter = None
status["runBlock"] = ps.ps6000aRunBlock(chandle, noOfPreTriggerSamples, noOfPostTriggerSamples, timebase, ctypes.byref(timeIndisposedMs), 0, None, None)
assert_pico_ok(status["runBlock"])

# Check for data collection to finish using ps6000aIsReady
ready = ctypes.c_int16(0)
check = ctypes.c_int16(0)
while ready.value == check.value:
    status["isReady"] = ps.ps6000aIsReady(chandle, ctypes.byref(ready))
    
# Get data from scope
# handle = chandle
# startIndex = 0
noOfSamples = ctypes.c_uint64(nSamples)
# downSampleRatio = 1
# segmentIndex = 0
# Creates a overflow location for each segment
overflow = (ctypes.c_int16 * 10)()
status["getValues"] = ps.ps6000aGetValuesBulk(chandle, 0, ctypes.byref(noOfSamples),0, 9, 1, downSampleMode, ctypes.byref(overflow))
assert_pico_ok(status["getValues"])

# get max ADC value
# handle = chandle
minADC = ctypes.c_int16()
maxADC = ctypes.c_int16()
status["getAdcLimits"] = ps.ps6000aGetAdcLimits(chandle, resolution, ctypes.byref(minADC), ctypes.byref(maxADC))
assert_pico_ok(status["getAdcLimits"])

# convert ADC counts data to mV
adc2mVChAMax =  adc2mV(bufferAMax, channelRange, maxADC)
adc2mVChAMax1 =  adc2mV(bufferAMax1, channelRange, maxADC)
adc2mVChAMax2 =  adc2mV(bufferAMax2, channelRange, maxADC)
adc2mVChAMax3 =  adc2mV(bufferAMax3, channelRange, maxADC)
adc2mVChAMax4 =  adc2mV(bufferAMax4, channelRange, maxADC)
adc2mVChAMax5 =  adc2mV(bufferAMax5, channelRange, maxADC)
adc2mVChAMax6 =  adc2mV(bufferAMax6, channelRange, maxADC)
adc2mVChAMax7 =  adc2mV(bufferAMax7, channelRange, maxADC)
adc2mVChAMax8 =  adc2mV(bufferAMax8, channelRange, maxADC)
adc2mVChAMax9 =  adc2mV(bufferAMax9, channelRange, maxADC)

# Create time data
time = np.linspace(0, (nSamples -1) * timeInterval.value * 1000000000, nSamples)

# plot data from channel A and B
plt.plot(time, adc2mVChAMax[:])
plt.plot(time, adc2mVChAMax1[:])
plt.plot(time, adc2mVChAMax2[:])
plt.plot(time, adc2mVChAMax3[:])
plt.plot(time, adc2mVChAMax4[:])
plt.plot(time, adc2mVChAMax5[:])
plt.plot(time, adc2mVChAMax6[:])
plt.plot(time, adc2mVChAMax7[:])
plt.plot(time, adc2mVChAMax8[:])
plt.plot(time, adc2mVChAMax9[:])
plt.xlabel('Time (ns)')
plt.ylabel('Voltage (mV)')
plt.show()

# Close the scope
status["closeunit"] = ps.ps6000aCloseUnit(chandle)
assert_pico_ok(status["closeunit"])

print(status)



from ctypes import POINTER, c_int16, c_uint32

from picosdk.ps2000 import ps2000
from picosdk.functions import assert_pico2000_ok
from picosdk.ctypes_wrapper import C_CALLBACK_FUNCTION_FACTORY

from enum import IntEnum
from concurrent.futures import ThreadPoolExecutor

import functools
import time
import os

CALLBACK = C_CALLBACK_FUNCTION_FACTORY(None, POINTER(POINTER(c_int16)), c_int16, c_uint32, c_int16, c_int16, c_uint32)

THREADPOOL = ThreadPoolExecutor()

past_mv_values = []

term_width = os.get_terminal_size()


def process_values(adc_values):
    mv_values = adc_to_mv(adc_values, PotentialRange.PS2000_50MV)

    for value in mv_values:
        pre = (value + 500) / 500 * term_width
        print('{}|'.format(' ' * pre))

    past_mv_values.extend(mv_values)


def get_overview_buffers(buffers, _overflow, _triggered_at, _triggered, _auto_stop, n_values):
    adc_values = buffers[0][0:n_values]

    THREADPOOL.submit(functools.partial(process_values, adc_values))


callback = CALLBACK(get_overview_buffers)


def adc_to_mv(values, range_, bitness=16):
    v_ranges = [10, 20, 50, 100, 200, 500, 1_000, 2_000, 5_000, 10_000, 20_000]

    return [(x * v_ranges[range_]) / (2**(bitness - 1) - 1) for x in values]


class Channel(IntEnum):
    PS2000_CHANNEL_A = 0
    PS2000_CHANNEL_B = 1


class PotentialRange(IntEnum):
    PS2000_20MV = 1
    PS2000_50MV = 2
    PS2000_100MV = 3
    PS2000_200MV = 4
    PS2000_500MV = 5
    PS2000_1V = 6
    PS2000_2V = 7
    PS2000_5V = 8
    PS2000_10V = 9
    PS2000_20V = 10


with ps2000.open_unit() as device:
    res = ps2000.ps2000_set_channel(device.handle, Channel.PS2000_CHANNEL_A, True, True, PotentialRange.PS2000_50MV)
    assert_pico2000_ok(res)

    res = ps2000.ps2000_run_streaming_ns(
        device.handle,
        500,
        2,
        100_000,
        False,
        1,
        50_000
    )
    assert_pico2000_ok(res)

    target_samples = 1_000_000

    while True:
        ps2000.ps2000_get_streaming_last_values(
            device.handle,
            callback
        )

        time.sleep(0.01)



# coding=utf-8
#
# Copyright (C) 2018 Pico Technology Ltd. See LICENSE file for terms.
#
"""
Definition of the Device class, which provides access to SDK functions which require a device handle, including
capturing data and configuring the AWG.
"""
from __future__ import print_function
import collections
import numpy
import math
import time
from picosdk.errors import DeviceCannotSegmentMemoryError, InvalidTimebaseError, ClosedDeviceError, \
    NoChannelsEnabledError, NoValidTimebaseForOptionsError


def requires_open(error_message="This operation requires a device to be connected."):
    def check_open_decorator(method):
        def check_open_impl(self, *args, **kwargs):
            if not self.is_open:
                raise ClosedDeviceError(error_message)
            return method(self, *args, **kwargs)
        return check_open_impl
    return check_open_decorator


"""ChannelConfig: A type for specifying channel setup for capture (pass into Device.set_channel, or Device.capture_*)
name = The channel name as a string (e.g. 'A'.)
enabled = bool indicating whether the channel should be enabled or disabled.
coupling (optional) = 'AC' or 'DC', default is 'DC'.
range_peak (optional) = +/- max volts, the highest precision range which includes your value will be selected.
analog_offset (optional) = the meaning of 0 for this channel."""
ChannelConfig = collections.namedtuple('ChannelConfig', ['name', 'enabled', 'coupling', 'range_peak', 'analog_offset'])
ChannelConfig.__new__.__defaults__ = (None, None, None)


"""TimebaseOptions: A type for specifying timebase constraints (pass to Device.find_timebase or Device.capture_*)
All are optional. Please specify the options which matter to you: 
  - the maximum time interval (if you want the fastest/most precise timebase you can get),
  - the number of samples in one buffer,
  - the minimum total collection time (if you want at least x.y seconds of uninterrupted capture data)
  - the oversample (if you want to sacrifice time precision for amplitude precision - see the programming guides.)"""
TimebaseOptions = collections.namedtuple('TimebaseOptions', ['max_time_interval',
                                                             'no_of_samples',
                                                             'min_collection_time',
                                                             'oversample'])
_TIMEBASE_OPTIONS_DEFAULTS = (None, None, None, 1)
TimebaseOptions.__new__.__defaults__ = _TIMEBASE_OPTIONS_DEFAULTS


class Device(object):
    """This object caches some information about the device state which cannot be queried from the driver. Please don't
    mix and match calls to this object with calls directly to the driver (or the ctypes wrapper), as this may cause
    unwanted behaviour (e.g. throwing an exception because no channels are enabled, when you enabled them yourself
    on the driver object.)"""
    def __init__(self, driver, handle):
        self.driver = driver
        self.handle = handle
        self.is_open = handle > 0

        # if a channel is missing from here, it is disabled (or in an undefined state).
        self._channel_ranges = {}
        self._channel_offsets = {}

    @requires_open("The device either did not initialise correctly or has already been closed.")
    def close(self):
        self.driver.close_unit(self)
        self.handle = None
        self.is_open = False

    @property
    @requires_open()
    def info(self):
        return self.driver.get_unit_info(self)

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.close()
        if all(i is None for i in args):
            return True
        return False

    @requires_open()
    def set_channel(self, channel_config):
        name = channel_config.name
        if not channel_config.enabled:
            self.driver.set_channel(self,
                                    channel_name=name,
                                    enabled=channel_config.enabled)
            try:
                del self._channel_ranges[name]
                del self._channel_offsets[name]
            except KeyError:
                pass
            return
        # if enabled, we pass through the values from the channel config:
        self._channel_ranges[name] = self.driver.set_channel(self,
                                                             channel_name=name,
                                                             enabled=channel_config.enabled,
                                                             coupling=channel_config.coupling,
                                                             range_peak=channel_config.range_peak,
                                                             analog_offset=channel_config.analog_offset)
        self._channel_offsets[name] = channel_config.analog_offset
        return self._channel_ranges[name]

    @requires_open()
    def set_channels(self, *channel_configs):
        """ set_channels(self, *channel_configs)
        An alternative to calling set_channel for each one, you can call this method with some channel configs.
        This method will also disable any missing channels from the passed configs, and disable ALL channels if the
        collection is empty. """
        # Add channels which are missing as "disabled".
        if len(channel_configs) < len(self.driver.PICO_CHANNEL):
            channel_configs = list(channel_configs)
            present_channels = set(c.name for c in channel_configs)
            missing_channels = [cn for cn in self.driver.PICO_CHANNEL.keys() if cn not in present_channels]
            for channel_name in missing_channels:
                channel_configs.append(ChannelConfig(channel_name, False))

        for channel_config in channel_configs:
            self.set_channel(channel_config)

    def _timebase_options_are_impossible(self, options):
        device_max_samples_possible = self.driver.MAX_MEMORY
        if options.no_of_samples is not None:
            if options.no_of_samples > device_max_samples_possible:
                return True
        elif options.max_time_interval is not None and options.min_collection_time is not None:
            effective_min_no_samples = math.ceil(options.min_collection_time / options.max_time_interval)
            if effective_min_no_samples > device_max_samples_possible:
                return True
        if None not in (options.no_of_samples, options.max_time_interval, options.min_collection_time):
            # if all 3 are requested, the result can be impossible.
            effective_min_no_samples = int(math.ceil(options.min_collection_time / options.max_time_interval))
            if effective_min_no_samples > options.no_of_samples:
                return True
        # Is it still possible that this device cannot handle this request, but we don't know without making calls to
        # get_timebase.
        return False

    @staticmethod
    def _validate_timebase(timebase_options, timebase_info):
        """validate whether a timebase result matches the options requested."""
        if timebase_options.max_time_interval is not None:
            if timebase_info.time_interval > timebase_options.max_time_interval:
                return False
        if timebase_options.no_of_samples is not None:
            if timebase_options.no_of_samples > timebase_info.max_samples:
                return False
        if timebase_options.min_collection_time is not None:
            if timebase_options.min_collection_time > timebase_info.max_samples * timebase_info.time_interval:
                return False
        return True

    @requires_open()
    def find_timebase(self, timebase_options):
        timebase_id = 0
        # quickly validate that the request is not impossible.
        if self._timebase_options_are_impossible(timebase_options):
            raise NoValidTimebaseForOptionsError()
        # TODO binary search?
        last_error = None
        found_one_good = False
        while True:
            try:
                timebase_info = self.driver.get_timebase(self, timebase_id, 0, timebase_options.oversample)
                found_one_good = True
                if self._validate_timebase(timebase_options, timebase_info):
                    return timebase_info
            except InvalidTimebaseError as e:
                if found_one_good:
                    # we won't find a valid timebase.
                    last_error = e
                    break
            timebase_id += 1
        args = ()
        if last_error is not None:
            args = (last_error.args[0],)
        raise NoValidTimebaseForOptionsError(*args)

    @requires_open()
    def capture_block(self, timebase_options, channel_configs=()):
        """device.capture_block(timebase_options, channel_configs)
        timebase_options: TimebaseOptions object, specifying at least 1 constraint, and optionally oversample.
        channel_configs: a collection of ChannelConfig objects. If present, will be passed to set_channels.
        """
        # set_channel:

        if channel_configs:
            self.set_channels(*channel_configs)

        if len(self._channel_ranges) == 0:
            raise NoChannelsEnabledError("We cannot capture any data if no channels are enabled.")

        # memory_segments:
        USE_SEGMENT_ID=0
        try:
            # always force the number of memory segments on the device to 1 before computing timebases for a one-off
            # block capture.
            max_samples_possible = self.driver.memory_segments(self, USE_SEGMENT_ID+1)
            if timebase_options.no_of_samples is not None and timebase_options.no_of_samples > max_samples_possible:
                raise NoValidTimebaseForOptionsError()
        except DeviceCannotSegmentMemoryError:
            pass

        # get_timebase
        timebase_info = self.find_timebase(timebase_options)

        post_trigger_samples = timebase_options.no_of_samples
        pre_trigger_samples = 0

        if post_trigger_samples is None:
            post_trigger_samples = int(math.ceil(timebase_options.min_collection_time / timebase_info.time_interval))

        self.driver.set_null_trigger(self)

        # tell the device to capture something:
        approx_time_busy = self.driver.run_block(self,
                                                 pre_trigger_samples,
                                                 post_trigger_samples,
                                                 timebase_info.timebase_id,
                                                 timebase_options.oversample,
                                                 USE_SEGMENT_ID)

        is_ready = self.driver.is_ready(self)
        while not is_ready:
            time.sleep(approx_time_busy / 5)
            is_ready = self.driver.is_ready(self)

        raw_data, overflow_warnings = self.driver.get_values(self,
                                                             self._channel_ranges.keys(),
                                                             post_trigger_samples,
                                                             USE_SEGMENT_ID)

        self.driver.stop(self)

        times = numpy.linspace(0.,
                               post_trigger_samples * timebase_info.time_interval,
                               post_trigger_samples,
                               dtype=numpy.dtype('float32'))

        voltages = {}

        max_adc = self.driver.maximum_value(self)
        for channel, raw_array in raw_data.items():
            array = raw_array.astype(numpy.dtype('float32'), casting='safe')
            factor = self._channel_ranges[channel] / max_adc
            array = array * factor
            voltages[channel] = array

        return times, voltages, overflow_warnings



from ctypes import byref, c_int16, c_int32, sizeof, Structure, c_uint16
from time import sleep

import numpy as np
from picosdk.ps2000 import ps2000
from picosdk.functions import assert_pico2000_ok, adc2mV
from picosdk.PicoDeviceEnums import picoEnum

import matplotlib.pyplot as plt

SAMPLES = 2000
OVERSAMPLING = 1
PRESAMPLE = 10.0
ADC_THRESHOLD = 8192

THRESHOLD_DIRECTION = {
    'PS2000_ABOVE': 0,
    'PS2000_BELOW': 1,
    'PS2000_ADV_RISING': 2,
    'PS2000_ADV_FALLING': 3,
    'PS2000_ADV_RISING_OR_FALLING': 4,
    'PS2000_INSIDE': 0,
    'PS2000_OUTSIDE': 1,
    'PS2000_ENTER': 2,
    'PS2000_EXIT': 3,
    'PS2000_ENTER_OR_EXIT': 4,
    'PS2000_ADV_NONE': 2,
}


class TriggerConditions(Structure):
    _fields_ = [
        ('channelA', c_int32),
        ('channelB', c_int32),
        ('channelC', c_int32),
        ('channelD', c_int32),
        ('external', c_int32),
        ('pulseWidthQualifier', c_int32),
    ]


class PwqConditions(Structure):
    _fields_ = [
        ('channelA', c_int32),
        ('channelB', c_int32),
        ('channelC', c_int32),
        ('channelD', c_int32),
        ('external', c_int32),
    ]


class TriggerChannelProperties(Structure):
    _fields_ = [
        ("thresholdMajor", c_int16),
        ("thresholdMinor", c_int16),
        ("hysteresis", c_uint16),
        ("channel", c_int16),
        ("thresholdMode", c_int16),
    ]


def get_timebase(device, wanted_time_interval):
    current_timebase = 1

    old_time_interval = None
    time_interval = c_int32(0)
    time_units = c_int16()
    max_samples = c_int32()

    while ps2000.ps2000_get_timebase(
        device.handle,
        current_timebase,
        SAMPLES,
        byref(time_interval),
        byref(time_units),
        1,
        byref(max_samples)) == 0 \
        or time_interval.value < wanted_time_interval:

        current_timebase += 1
        old_time_interval = time_interval.value

        if current_timebase.bit_length() > sizeof(c_int16) * 8:
            raise Exception('No appropriate timebase was identifiable')

    return current_timebase - 1, old_time_interval


with ps2000.open_unit() as device:
    print('Device info: {}'.format(device.info))

    res = ps2000.ps2000_set_channel(
        device.handle,
        picoEnum.PICO_CHANNEL['PICO_CHANNEL_A'],
        True,
        picoEnum.PICO_COUPLING['PICO_DC'],
        ps2000.PS2000_VOLTAGE_RANGE['PS2000_50MV']
    )
    assert_pico2000_ok(res)

    trigger_conditions = TriggerConditions(1, 0, 0, 0, 0, 1)

    res = ps2000.ps2000SetAdvTriggerChannelConditions(device.handle, byref(trigger_conditions), 1)
    assert_pico2000_ok(res)

    res = ps2000.ps2000SetAdvTriggerDelay(device.handle, 0, -PRESAMPLE)
    assert_pico2000_ok(res)

    res = ps2000.ps2000SetAdvTriggerChannelDirections(
        device.handle,
        THRESHOLD_DIRECTION['PS2000_ADV_RISING_OR_FALLING'],
        THRESHOLD_DIRECTION['PS2000_ADV_NONE'],
        THRESHOLD_DIRECTION['PS2000_ADV_NONE'],
        THRESHOLD_DIRECTION['PS2000_ADV_NONE'],
        THRESHOLD_DIRECTION['PS2000_ADV_NONE']
    )
    assert_pico2000_ok(res)

    trigger_properties = TriggerChannelProperties(
        ADC_THRESHOLD,
        ADC_THRESHOLD,
        328,
        ps2000.PS2000_CHANNEL['PS2000_CHANNEL_A'],
        picoEnum.PICO_THRESHOLD_MODE['PICO_LEVEL']
    )

    res = ps2000.ps2000SetAdvTriggerChannelProperties(device.handle, byref(trigger_properties), 1, 0)
    assert_pico2000_ok(res)

    pwq_conditions = PwqConditions(1, 0, 0, 0, 0)

    res = ps2000.ps2000SetPulseWidthQualifier(
        device.handle,
        byref(pwq_conditions),
        1,
        THRESHOLD_DIRECTION['PS2000_ADV_RISING_OR_FALLING'],
        1000,
        0,
        picoEnum.PICO_PULSE_WIDTH_TYPE['PICO_PW_TYPE_GREATER_THAN']
    )
    assert_pico2000_ok(res)

    timebase_a, interval = get_timebase(device, 20_000)

    collection_time = c_int32()

    fig, ax = plt.subplots()
    ax.set_xlabel('time/ms')
    ax.set_ylabel('ADC counts')

    res = ps2000.ps2000_run_block(
        device.handle,
        SAMPLES,
        timebase_a,
        OVERSAMPLING,
        byref(collection_time)
    )
    assert_pico2000_ok(res)

    while ps2000.ps2000_ready(device.handle) == 0:
        sleep(0.1)

    times = (c_int32 * SAMPLES)()

    buffer_a = (c_int16 * SAMPLES)()

    res = ps2000.ps2000_get_times_and_values(
        device.handle,
        byref(times),
        byref(buffer_a),
        None,
        None,
        None,
        None,
        2,
        SAMPLES,
    )
    assert_pico2000_ok(res)

    channel_a_mv = adc2mV(buffer_a, ps2000.PS2000_VOLTAGE_RANGE['PS2000_50MV'], c_int16(32767))

    ax.plot(list(map(lambda x: x * 1e-6, times[:])), buffer_a[:])
    ax.plot(list(map(lambda x: x * 1e-6, times[:])), [ADC_THRESHOLD for _ in times[:]], color='red')
    ax.axvline(np.percentile(list(map(lambda x: x * 1e-6, times[:])), PRESAMPLE), color='red', linestyle='dotted')

    ps2000.ps2000_stop(device.handle)

    plt.show()



#
# Copyright (C) 2017-2018 Pico Technology Ltd.
#
from __future__ import print_function
from setuptools import setup

import ctypes
from ctypes import *
from ctypes.util import find_library
import sys
import os.path


signalfile = ".sdkwarning"
if not os.path.exists(signalfile):
    name = 'ps2000'
    try:
        if sys.platform == 'win32':
            result = ctypes.WinDLL(find_library(name))
        else:
            result = cdll.LoadLibrary(find_library(name))
    except OSError:
        print("Please install the PicoSDK in order to use this wrapper."
              "Visit https://www.picotech.com/downloads")
        exit(1)
    open(signalfile, 'a').close()

with open("README.md") as f:
    readme = f.read()

setup(
    name='picosdk',
    packages=['picosdk'],
    install_requires=["numpy>=1.12.1"],
    version='1.0',
    description='PicoSDK Python wrapper',
    long_description=readme,
    author='Pico Technology Ltd',
    author_email='support@picotech.com',
    license="ISC",
    url='https://www.picotech.com',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: ISC License (ISCL)",
        "Framework :: Matplotlib",
        "Natural Language :: English",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Scientific/Engineering",
    ],
    universal=True,
)



#
# Copyright (C) 2018 Pico Technology Ltd. See LICENSE file for terms.
#
# PicoScope 6000 Series Signal Generator Example
# This example demonstrates how to use the PicoScope 6000 Series (ps6000) driver API functions to to set up the signal generator to do the following:
# 
# 1. Output a sine wave 
# 2. Output a square wave 
# 3. Output a sweep of a square wave signal

import ctypes
from picosdk.ps6000 import ps6000 as ps
import time
from picosdk.functions import assert_pico_ok


# Gives the device a handle
status = {}
chandle = ctypes.c_int16()

# Opens the device/s
status["openunit"] = ps.ps6000OpenUnit(ctypes.byref(chandle), None)
assert_pico_ok(status["openunit"])

# Output a sine wave with peak-to-peak voltage of 2 V and frequency of 10 kHz
# handle = chandle
# offsetVoltage = 0
# pkToPk = 2000000
# waveType = ctypes.c_int16(0) = PS6000_SINE
# startFrequency = 10 kHz
# stopFrequency = 10 kHz
# increment = 0
# dwellTime = 1
# sweepType = ctypes.c_int16(1) = PS6000_UP
# operation = 0
# shots = 0
# sweeps = 0
# triggerType = ctypes.c_int16(0) = PS6000_SIGGEN_RISING
# triggerSource = ctypes.c_int16(0) = PS6000_SIGGEN_NONE
# extInThreshold = 1
wavetype = ctypes.c_int16(0)
sweepType = ctypes.c_int32(0)
triggertype = ctypes.c_int32(0)
triggerSource = ctypes.c_int32(0)

status["SetSigGenBuiltIn"] = ps.ps6000SetSigGenBuiltIn(chandle, 0, 2000000, wavetype, 10000, 10000, 0, 1, sweepType, 0, 0, 0, triggertype, triggerSource, 1)
assert_pico_ok(status["SetSigGenBuiltIn"])

# Pauses the script to show signal
time.sleep(10)

# Output a square wave with peak-to-peak voltage of 2 V and frequency of 10 kHz
# handle = chandle
# offsetVoltage = -1000000
# pkToPk = 1500000
# waveType = ctypes.c_int16(1) = PS6000_SQUARE
# startFrequency = 10 kHz
# stopFrequency = 10 kHz
# increment = 0
# dwellTime = 1
# sweepType = ctypes.c_int16(1) = PS6000_UP
# operation = 0
# shots = 0
# sweeps = 0
# triggerType = ctypes.c_int16(0) = PS6000_SIGGEN_RISING
# triggerSource = ctypes.c_int16(0) = PS6000_SIGGEN_NONE
# extInThreshold = 1
wavetype = ctypes.c_int16(1)
sweepType = ctypes.c_int32(0)
triggertype = ctypes.c_int32(0)
triggerSource = ctypes.c_int32(0)

status["SetSigGenBuiltIn"] = ps.ps6000SetSigGenBuiltIn(chandle, 0, 2000000, wavetype, 10000, 10000, 0, 1, sweepType, 0, 0, 0, triggertype, triggerSource, 1)
assert_pico_ok(status["SetSignGenBuiltIn"])

# Pauses the script to show signal
time.sleep(10)

# Output a square wave with an up-down sweep, 10-100 kHz in 5 kHz increments every 1 second.
# handle = chandle
# offsetVoltage = -1000000
# pkToPk = 1500000
# waveType = ctypes.c_int16(1) = PS6000_SQUARE
# startFrequency = 10 kHz
# stopFrequency = 100 kHz
# increment = 5 kHz
# dwellTime = 1
# sweepType = ctypes.c_int16(1) = PS6000_UP
# operation = 0
# shots = 0
# sweeps = 0
# triggerType = ctypes.c_int16(0) = PS6000_SIGGEN_RISING
# triggerSource = ctypes.c_int16(0) = PS6000_SIGGEN_NONE
# extInThreshold = 1
wavetype = ctypes.c_int16(1)
sweepType = ctypes.c_int32(2)
triggertype = ctypes.c_int32(0)
triggerSource = ctypes.c_int32(0)

status["SetSigGenBuiltIn"] = ps.ps6000SetSigGenBuiltIn(chandle, 0, 2000000, wavetype, 10000, 100000, 5000, 1, sweepType, 0, 0, 0, triggertype, triggerSource, 1)
assert_pico_ok(status["SetSigGenBuiltIn"])

# Pauses the script to show signal
time.sleep(36)

# Closes the unit
# Handle = chandle
status["close"] = ps.ps6000CloseUnit(chandle)
assert_pico_ok(status["close"])

# Displays the status returns
print(status)


#
# Copyright (C) 2020 Pico Technology Ltd. See LICENSE file for terms.
#
"""
This is a Python module defining the structures from the PicoDeviceStructs.h C header
file for use with PicoScope 6000 A Series oscilloscopes using the ps6000a driver API
functions.
"""

from ctypes import *
from picosdk.library import Library

class PicoStructlib(Library):
    def __init__(self):
        super(PicoStructlib, self).__init__("ps6000a")


picoStruct = PicoStructlib()

class PICO_TRIGGER_INFO(Structure):
    _pack_ = 1
    _fields_ = [("status", c_uint32),
                ("segmentIndex", c_uint64),
                ("triggerIndex", c_uint64),
                ("triggerTime", c_double),
                ("timeUnits", c_uint32),
                ("missedTriggers", c_uint64),
                ("timeStampCounter", c_uint64)]
                
picoStruct.PICO_TRIGGER_INFO = PICO_TRIGGER_INFO

class PICO_TRIGGER_CHANNEL_PROPERTIES(Structure):
    _pack_ = 1
    _fields_ = [("thresholdUpper", c_int16),
                ("thresholdUpperHysteresis", c_uint16),
                ("thresholdLower", c_int16),
                ("thresholdLowerHysteresis", c_uint16),
                ("channel", c_uint32)]
                
picoStruct.PICO_TRIGGER_CHANNEL_PROPERTIES = PICO_TRIGGER_CHANNEL_PROPERTIES

class PICO_CONDITION(Structure):
    _pack_ = 1
    _fields_ = [("source", c_uint32),
                ("condition", c_uint32)]
                
picoStruct.PICO_CONDITION = PICO_CONDITION

class PICO_DIRECTION(Structure):
    _pack_ = 1
    _fields_ = [("channel", c_uint32),
                ("direction", c_uint32),
                ("thresholdMode", c_uint32)]
                
picoStruct.PICO_DIRECTION = PICO_DIRECTION

class PICO_USER_PROBE_INTERACTIONS(Structure):
    _pack_ = 1
    _fields_ = [("connected", c_uint16),
                ("channel", c_uint32),
                ("enabled", c_uint16),
                ("probeName", c_uint32),
                ("requiresPower", c_uint8),
                ("isPowered", c_uint8),
                ("status", c_uint32),
                ("probeOff", c_uint32),
                ("rangeFirst", c_uint32),
                ("rangeLast", c_uint32),
                ("rangeCurrent", c_uint32),
                ("couplingFirst", c_uint32),
                ("couplingLast", c_uint32),
                ("couplingCurrent", c_uint32),
                ("filterFlags", c_uint32),
                ("filterCurrent", c_uint32)]
                
picoStruct.PICO_USER_PROBE_INTERACTIONS = PICO_USER_PROBE_INTERACTIONS

class PICO_DATA_BUFFERS(Structure):
    _pack_ = 1
    _fields_ = [("channel", c_uint32),
                ("waveform", c_uint64),
                ("downSampleRatioMode", c_uint32),
                ("read", c_uint32),
                ("bufferMax", c_void_p),
                ("bufferMin", c_void_p),
                ("dataType", c_uint32),
                ("nDistributionPoints", c_uint32)]
                
picoStruct.PICO_DATA_BUFFERS = PICO_DATA_BUFFERS

class PICO_STREAMING_DATA_INFO(Structure):
    _pack_ = 1
    _fields_ = [("channel", c_uint32),
                ("mode", c_uint32),
                ("type", c_uint32),
                ("noOfSamples", c_int32),
                ("bufferIndex", c_uint64),
                ("startIndex", c_int32),
                ("overflow", c_int16)]
                
picoStruct.PICO_STREAMING_DATA_INFO = PICO_STREAMING_DATA_INFO

class PICO_STREAMING_DATA_TRIGGER_INFO(Structure):
    _pack_ = 1
    _fields_ = [("triggerAt", c_uint64),
                ("triggered", c_int16),
                ("autoStop", c_int16)]
                
picoStruct.PICO_STREAMING_DATA_TRIGGER_INFO = PICO_STREAMING_DATA_TRIGGER_INFO

class PICO_SCALING_FACTORS(Structure):
    _pack_ = 1
    _fields_ = [("channel", c_uint32),
                ("range", c_uint32),
                ("offset", c_int16),
                ("scalingFactor", c_double)]
                
picoStruct.PICO_SCALING_FACTORS = PICO_SCALING_FACTORS

class PROBE_APP(Structure):
    _pack_ = 1
    _fields_ = [("id", c_int32),
                ("appMajorVersion", c_int32),
                ("appMinorVersion", c_int32)]
               
picoStruct.PROBE_APP = PROBE_APP

class DIGITAL_CHANNEL_DIRECTIONS(Structure):
    _pack_ = 1
    _fields_ = [("channel", c_uint32),
                ("direction", c_uint32)]
                
picoStruct.DIGITAL_CHANNEL_DIRECTIONS = DIGITAL_CHANNEL_DIRECTIONS

class PICO_DIGITAL_PORT_INTERACTIONS(Structure):
    _pack_ = 1
    _fields_ = [("connected", c_uint16),
                ("channel", c_uint32),
                ("digitalPortName", c_uint32),
                ("status", c_uint32),
                ("serial", c_int8),
                ("calibrationDate", c_int8)]
                
picoStruct.PICO_DIGITAL_PORT_INTERACTIONS = PICO_DIGITAL_PORT_INTERACTIONS


#
# Copyright (C) 2018 Pico Technology Ltd. See LICENSE file for terms.
#
from __future__ import print_function
from picosdk.discover import find_unit
from picosdk.device import ChannelConfig, TimebaseOptions
import matplotlib.pyplot as plt

with find_unit() as device:

    print("found PicoScope: %s" % (device.info,))

    channel_configs = [ChannelConfig('A', True, 'DC', 5.)]
    microsecond = 1.e-6
    # the entry-level scopes only have about 8k-samples of memory onboard for block mode, so only ask for 6k samples.
    timebase_options = TimebaseOptions(microsecond, None, 6000 * microsecond)

    times, voltages, overflow_warnings = device.capture_block(timebase_options, channel_configs)

    for channel, data in voltages.items():
        label = "Channel %s" % channel
        if channel in overflow_warnings:
            label += " (over range)"
        plt.plot(times, data, label=label)

    plt.xlabel('Time / s')
    plt.ylabel('Amplitude / V')
    plt.legend()
    plt.show()




#
# Copyright (C) 2019 Pico Technology Ltd. See LICENSE file for terms.
#
"""
This is a Python module defining the functions from the picohrdl.h C header file
for ADC-20/24 Data Loggers using the picohrdl driver API functions.
"""

from ctypes import *
from picosdk.library import Library
from picosdk.errors import ArgumentOutOfRangeError
from picosdk.constants import make_enum

class picohrdllib(Library):
    def __init__(self):
        super(picohrdllib, self).__init__("picohrdl")
		
picohrdl = picohrdllib()

picohrdl.HRDL_VOLTAGERANGE = make_enum([
    "HRDL_2500_MV",
	"HRDL_1250_MV",
	"HRDL_625_MV",
	"HRDL_313_MV",
	"HRDL_156_MV",
	"HRDL_78_MV",
	"HRDL_39_MV",
])

picohrdl.HRDL_CONVERSIONTIME = make_enum([
    "HRDL_60MS",
	"HRDL_100MS",
	"HRDL_180MS",
	"HRDL_340MS",
	"HRDL_660MS",
])

doc = """ int16_t HRDLCloseUnit
    (
	    int16_t    handle
	); """
picohrdl.make_symbol("_closeUnit_", "HRDLCloseUnit", c_int16, [c_int16], doc)

doc = """ int16_t HRDLCollectSingleValueAsync
    (
	    int16_t    handle,
		int16_t    channel,
		int16_t    range,
		int16_t    conversionTime,
		int16_t    singleEnded
	); """
picohrdl.make_symbol("_collectSingleValue_Async_", "HRDLCollectSingleValueAsync", c_int16, [c_int16, c_int16, c_int16, c_int16, c_int16], doc)

doc = """ int16_t HRDLGetMinMaxAdcCounts
    (
	    int16_t    handle,
		int32_t    *minAdc,
		int32_t    *maxAdc,
		int16_t    channel
	); """
picohrdl.make_symbol("_getMinMaxAdcCounts_", "HRDLGetMinMaxAdcCounts", c_int16, [c_int16, c_void_p, c_void_p, c_int16], doc)

doc = """ int16_t HRDLGetNumberOfEnabledChannels
    (
	    int16_t    handle,
		int16_t    *nEnabledChannels
	); """
picohrdl.make_symbol("_getNumberOfEnabledChannels_", "HRDLGetNumberOfEnabledChannels", c_int16, [c_int16, c_void_p], doc)

doc = """ int16_t HRDLGetSingleValue
    (
	    int16_t    handle,
		int16_t    channel,
		int16_t    range,
		int16_t    conversionTime,
		int16_t    singleEnded,
		int16_t    *overflow,
		int32_t    *value
	); """
picohrdl.make_symbol("_getSingleValue_", "HRDLGetSingleValue", c_int16, [c_int16, c_int16, c_int16, c_int16, c_int16, c_void_p, c_void_p], doc)

doc = """ int16_t HRDLGetSingleValueAsync
    (
	    int16_t    handle,
		int32_t    *value,
		int16_t    *overflow
	); """
picohrdl.make_symbol("_getSingleValueAsync_", "HRDLGetSingleValueAsync", c_int16, [c_int16, c_void_p, c_void_p], doc)

doc = """ int32_t HRDLGetTimesAndVAlues
    (
	    int16_t    handle,
		int32_t    *times,
		int32_t    *values,
		int16_t    *overflow,
		int32_t    noOfValues
	); """
picohrdl.make_symbol("_getTimesAndValues_", "HRDLGetTimesAndValues", c_int16, [c_int16, c_void_p, c_void_p, c_void_p, c_int32], doc)

doc = """ int16_t HRDLGetUnitInfo
    (
	    int16_t    handle,
		int8_t     *string,
		int16_t    stringLength,
		int16_t    info
	); """
picohrdl.make_symbol("_getUnitInfo_", "HRDLGetUnitInfo", c_int16, [c_int16, c_void_p, c_int16, c_int16], doc)

doc = """ int32_t HRDLGetValues
    (
	    int16_t    handle,
		int32_t    *values,
		int16_t    *overflow,
		int32_t    noOfValues
	); """
picohrdl.make_symbol("_getValues_", "HRDLGetValues", c_int16, [c_int16, c_void_p, c_void_p, c_int32], doc)

doc = """ int16_t HRDLOpenUnit
    (
	    void
	); """
picohrdl.make_symbol("_openUnit_", "HRDLOpenUnit", c_int16, [], doc)

doc = """ int16_t HRDLOpenUnitAsync
    (
	    void
	); """
picohrdl.make_symbol("_openUnitAsync_", "HRDLOpenUnitAsync", c_int16, [], doc)

doc = """ int16_t HRDLOpenUnitProgress
    (
	    int16_t    *handle,
		int16_t    *progress
	); """
picohrdl.make_symbol("_openUnitProgress_", "HRDLOpenUnitProgress", c_int16, [c_void_p, c_void_p], doc)

doc = """ int16_t HRDLReady
    (
	    int16_t    handle
	); """
picohrdl.make_symbol("_ready_", "HRDLReady", c_int16, [c_int16], doc)

doc = """ int16_t HRDLRun
    (
	    int16_t    handle,
		int32_t    nValues,
		int16_t    method
	); """
picohrdl.make_symbol("_run_", "HRDLRun", c_int16, [c_int16, c_int32, c_int16], doc)

doc = """ int16_t HRDLSetAnalogInChannel
    (
	    int16_t    handle,
		int16_t    channel,
		int16_t    enabled,
		int16_t    range,
		int16_t    singleEnded
	); """
picohrdl.make_symbol("_setAnalogInChannel_", "HRDLSetAnalogInChannel", c_int16, [c_int16, c_int16, c_int16, c_int16, c_int16], doc)

doc = """ int16_t HRDLSetDigitalIOChannel
    (
	    int16_t    handle,
		int16_t    directionOut,
		int16_t    digitalOutPinState,
		int16_t    enabledDigitalIn
	); """
picohrdl.make_symbol("_setDigitalIOChannel_", "HRDLSetDigitalIOChannel", c_int16, [c_int16, c_int16, c_int16, c_int16], doc)

doc = """ int16_t HRDLSetInterval
    (
	    int16_t    handle,
		int32_t    samplesInterval_ms,
		int16_t    conversionTime
	); """
picohrdl.make_symbol("_setInterval_", "HRDLSetInterval", c_int16, [c_int16, c_int32, c_int16], doc)

doc = """ int16_t HRDLSetMains
    (
	    int16_t    handle,
		int16_t    sixtyHertz
	); """
picohrdl.make_symbol("_setMains_", "HRDLSetMains", c_int16, [c_int16, c_int16], doc)

doc = """ void HRDLStop
    (
	    int16_t    handle
	); """
picohrdl.make_symbol("_stop_", "HRDLStop", c_int16, [c_int16], doc)


#
# Copyright (C) 2020 Pico Technology Ltd. See LICENSE file for terms.
#
# PS6000 A SIGNAL GENERATOR EXAMPLE
# This example opens a 6000a driver device, sets up the signal generator to do the following:
# 
# 1. Output a sine wave 
# 2. Output a square wave 
# 3. Output a sweep of a square wave signal

import ctypes
import numpy as np
from picosdk.ps6000a import ps6000a as ps
from picosdk.PicoDeviceEnums import picoEnum as enums
from picosdk.functions import adc2mV, assert_pico_ok
import time

# Create chandle and status ready for use
chandle = ctypes.c_int16()
status = {}

# Open 6000 A series PicoScope
# returns handle to chandle for use in future API functions
resolution = enums.PICO_DEVICE_RESOLUTION["PICO_DR_8BIT"]
status["openunit"] = ps.ps6000aOpenUnit(ctypes.byref(chandle), None, resolution)
assert_pico_ok(status["openunit"])

# Set signal generator waveform
# handle = chandle
wavetype = enums.PICO_WAVE_TYPE["PICO_SINE"]
bufferlength = 1000
buffer = (ctypes.c_int16 * bufferlength)()
status["sigGenWaveform"] = ps.ps6000aSigGenWaveform(chandle, wavetype, ctypes.byref(buffer), bufferlength)
assert_pico_ok(status["sigGenWaveform"])

# Set signal generator range
# handle = chandle
peakToPeakVolts = 2
offsetVolts = 0
status["sigGenRange"] = ps.ps6000aSigGenRange(chandle, peakToPeakVolts, offsetVolts)
assert_pico_ok(status["sigGenRange"])

# Set signal generator duty cycle
# handle = chandle
dutyCyclePercent = 50
status["sigGenDutyCycle"] = ps.ps6000aSigGenWaveformDutyCycle(chandle, dutyCyclePercent)
assert_pico_ok(status["sigGenDutyCycle"])

# Set signal generator frequency
# handle = chandle
frequencyHz = 1000
status["sigGenFreq"] = ps.ps6000aSigGenFrequency(chandle, frequencyHz)
assert_pico_ok(status["sigGenFreq"])

# Apply signal generator settings
# handle = chandle
sigGenEnabled = 1
sweepEnabled = 0
triggerEnabled = 0
automaticClockOptimisationEnabled = 0
overrideAutomaticClockAndPrescale = 0
frequency = ctypes.c_int16(frequencyHz)
#stopFrequency = None
#frequencyIncrement = None
#dwellTime = None
status["sigGenApply"] = ps.ps6000aSigGenApply(chandle, 
                                              sigGenEnabled, 
                                              sweepEnabled, 
                                              triggerEnabled, 
                                              automaticClockOptimisationEnabled, 
                                              overrideAutomaticClockAndPrescale, 
                                              ctypes.byref(frequency), 
                                              None,
                                              None,
                                              None
                                              )
assert_pico_ok(status["sigGenApply"])

time.sleep(5)

# Pause signal generator
# handle = chandle
status["sigGenPause"] = ps.ps6000aSigGenPause(chandle)
assert_pico_ok(status["sigGenPause"])

time.sleep(5)

# Resume signal generator
# handle = chandle
status["sigGenRestart"] = ps.ps6000aSigGenRestart(chandle)
assert_pico_ok(status["sigGenRestart"])

time.sleep(5)

# Pause signal generator
# handle = chandle
status["sigGenPause"] = ps.ps6000aSigGenPause(chandle)
assert_pico_ok(status["sigGenPause"])

# Close the scope
status["closeunit"] = ps.ps6000aCloseUnit(chandle)
assert_pico_ok(status["closeunit"])

print(status)


#
# Copyright (C) 2014-2020 Pico Technology Ltd. See LICENSE file for terms.
#
"""
This is a Python module defining the functions from the usbDrDaqApi.h C header
file for DrDaq Data Logger using the usbDrDaq driver API
functions.
"""

from ctypes import *
from picosdk.ctypes_wrapper import C_CALLBACK_FUNCTION_FACTORY
from picosdk.library import Library
from picosdk.constants import make_enum

class UsbDrDaqlib(Library):
    def __init__(self):
        super(UsbDrDaqlib, self).__init__("usbDrDaq")


usbDrDaq = UsbDrDaqlib()

usbDrDaq.USB_DRDAQ_INPUTS = {
    "USB_DRDAQ_CHANNEL_EXT1":	1,
	"USB_DRDAQ_CHANNEL_EXT2":	2,
	"USB_DRDAQ_CHANNEL_EXT3":	3,
	"USB_DRDAQ_CHANNEL_SCOPE":	4,
	"USB_DRDAQ_CHANNEL_PH":		5,
	"USB_DRDAQ_CHANNEL_RES":	6,
	"USB_DRDAQ_CHANNEL_LIGHT":	7,
	"USB_DRDAQ_CHANNEL_TEMP":	8,
	"USB_DRDAQ_CHANNEL_MIC_WAVE":	9,
	("USB_DRDAQ_CHANNEL_MIC_LEVEL", "USB_DRDAQ_MAX_CHANNELS"):	10,
}

usbDrDaq.USB_DRDAQ_GPIO = {
    "USB_DRDAQ_GPIO_1":1,
	"USB_DRDAQ_GPIO_2":2,
	"USB_DRDAQ_GPIO_3":3,
	"USB_DRDAQ_GPIO_4":4
}

usbDrDaq.USB_DRDAQ_SCOPE_RANGE = make_enum([
    "USB_DRDAQ_1V25",
	"USB_DRDAQ_2V5",
	"USB_DRDAQ_5V",
	"USB_DRDAQ_10V"
])

usbDrDaq.USB_DRDAQ_WAVE = make_enum([
    "USB_DRDAQ_SINE",
	"USB_DRDAQ_SQUARE",
	"USB_DRDAQ_TRIANGLE",
	"USB_DRDAQ_RAMP_UP",
	"USB_DRDAQ_RAMP_DOWN",
	"USB_DRDAQ_DC"
])

usbDrDaq.USB_DRDAQ_BLOCK_METHOD = make_enum([
    "BM_SINGLE",
	"BM_WINDOW",
	"BM_STREAM"
])

doc = """ PICO_STATUS UsbDrDaqCloseUnit
    (
	    int16_t    handle
	); """
usbDrDaq.make_symbol("_CloseUnit", "UsbDrDaqCloseUnit", c_uint32, [c_int16], doc)

doc = """ PICO_STATUS UsbDrDaqEnabledRGBLED
    (
	    int16_t    handle,
		int16_t    enabled
	); """
usbDrDaq.make_symbol("_EnableRGBLED", "UsbDrDaqEnableRGBLED", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS UsbDrDaqGetChannelInfo
    (
	    int16_t    handle,
		float      *min,
		float      *max,
		int16_t    *places,
		int16_t    *divider,
		USB_DRDAQ_INPUTS    channel
	); """
usbDrDaq.make_symbol("_GetChannelInfo", "UsbDrDaqGetChannelInfo", c_uint32, [c_int16, c_void_p, c_void_p, c_void_p, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS UsbDrDaqGetInput
    (
	    int16_t    handle,
		USB_DRDAQ_GPIO    IOChannel,
		int16_t    pullUp,
		int16_t    *value
	); """
usbDrDaq.make_symbol("_GetInput", "UsbDrDaqGetInput", c_uint32, [c_int16, c_uint32, c_int16, c_void_p], doc)

doc = """ PICO_STATUS UsbDrDaqGetPulseCount
    (
	    int16_t    handle,
		USB_DRDAQ_GPIO    IOChannel,
		int16_t    *count
	); """
usbDrDaq.make_symbol("_GetPulseCount", "UsbDrDaqGetPulseCount", c_uint32, [c_int16, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS UsbDrDaqGetScalings
    (
	    int16_t    handle,
		USB_DRDAQ_INPUTS    channel,
		int16_t    *nScales
		int16_t    *currentScale,
		int8_t     *names
		int16_t    namesSize
	); """
usbDrDaq.make_symbol("_GetScalings", "UsbDrDaqGetScalings", c_uint32, [c_int16, c_uint32, c_void_p, c_void_p, c_void_p, c_int16], doc)

doc = """ PICO_STATUS UsbDrDaqGetSingle
    (
	    int16_t    handle,
		USB_DRDAQ_INPUTS    channel,
		int16_t    *value,
		uint16_t    *overflow
	); """
usbDrDaq.make_symbol("_GetSingle", "UsbDrDaqGetSingle", c_uint32, [c_int16, c_uint32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS UsbDrDaqGetSingleF
    (
	    int16_t    handle,
		USB_DRDAQ_INPUTS    channel,
		float    *value,
		uint16_t    *overflow
	); """
usbDrDaq.make_symbol("_getSingleF", "UsbDrDaqGetSingleF", c_uint32, [c_int16, c_uint32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS UsbDrDaqGetTriggerTimeOffestNs
    (
	    int16_t    handle,
		int64_t    *time
    ); """
usbDrDaq.make_symbol("_GetTriggerTimeOffsetNs", "UsbDrDaqGetTriggerTimeOffsetNs", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS UsbDrDaqGetUnitInfo
    (
	    int16_t    handle,
		int8_t    *string,
		int16_t     stringLength,
		int16_t     *requiredSize,
		PICO_INFO     info
	); """
usbDrDaq.make_symbol("_GetUnitInfo", "UsbDrDaqGetUnitInfo", c_uint32, [c_int16, c_void_p, c_int16, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS UsbDrDaqGetValues
    (
	    int16_t    handle,
		int16_t    *values,
		uint32_t    *noOfValues,
		uint16_t    *overflow,
		uint32_t    *triggerIndex
	); """
usbDrDaq.make_symbol("_GetValues", "UsbDrDaqGetValues", c_uint32, [c_int16, c_void_p, c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS UsbDrDaqGetValuesF
    (
	    int16_t    handle,
		float     *values,
		uint32_t    *noOfValues,
		uint16_t    *overflow,
		uint32_t    *triggerIndex
	); """
usbDrDaq.make_symbol("_GetValuesF", "UsbDrDaqGetValuesF", c_uint32, [c_int16, c_void_p, c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS UsbDrDaqOpenUnit
    (
	    int16_t    *handle
	); """
usbDrDaq.make_symbol("_OpenUnit", "UsbDrDaqOpenUnit", c_uint32, [c_void_p], doc)

doc = """ PICO_STATUS UsbDrDaqOpenUnitAsync
    (
	    int16_t    *handle
	); """
usbDrDaq.make_symbol("_OpenUnitAsync", "UsbDrDaqOpenUnitAsync", c_uint32, [c_void_p], doc)

doc = """ PICO_STATUS UsbDrDaqOpenUnitProgress
    (
	    int16_t    *handle,
		int16_t    *progress,
		int16_t    *complete
	); """
usbDrDaq.make_symbol("_OpenUnitProgress", "UsbDrDaqOpenUnitProgress", c_uint32, [c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS UsbDrDaqPhTemperatureCompensation
    (
	    int16_t    handle,
		uint16_t    enabled
	); """
usbDrDaq.make_symbol("_PhTemperatureCompensation", "UsbDrDaqPhTemperatureCompensation", c_uint32, [c_int16, c_uint16], doc)

doc = """ PICO_STATUS UsbDrDaqPingUnit
    (
	    int16_t    *handle
	); """
usbDrDaq.make_symbol("_PingUnit", "UsbDrDaqPingUnit", c_uint32, [c_void_p], doc)

doc = """ PICO_STATUS UsbDrDaqReady
    (
	    int16_t    handle,
		int16_t    *ready
	); """
usbDrDaq.make_symbol("_Ready", "UsbDrDaqReady", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS UsbDrDaqRun
    (
	    int16_t    handle,
		uint32_t    no_of_values,
		BLOCK_METHOD    method
	); """
usbDrDaq.make_symbol("_Run", "UsbDrDaqRun", c_uint32, [c_int16, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS UsbDrDaqSetDO
    (
	    int16_t    handle,
		USB_DRDAQ_GPIO    IOChannel,
		int16_t    value
	); """
usbDrDaq.make_symbol("_SetDO", "UsbDrDaqSetDO", c_uint32, [c_int16, c_uint32, c_int16], doc)

doc = """ PICO_STATUS UsbDrDaqSetInterval
    (
	    int16_t    handle,
		uint32_t    *us_for_block,
		uint32_t    ideal_no_of_samples,
		USB_DRDAQ_INPUTS    *channels,
		int16_t    no_of_channels
	); """
usbDrDaq.make_symbol("_SetInterval", "UsbDrDaqSetInterval", c_uint32, [c_int16, c_void_p, c_uint32, c_void_p, c_int16], doc)

doc = """ PICO_STATUS UsbDrDaqSetIntervalF
    (
	    int16_t    handle,
		float    *us_for_block,
		uint32_t    ideal_no_of_samples,
		USB_DRDAQ_INPUTS    *channels,
		int16_t    no_of_channels
	); """
usbDrDaq.make_symbol("_SetIntervalF", "UsbDrDaqSetIntervalF", c_uint32, [c_int16, c_void_p, c_uint32, c_void_p, c_int16], doc)

doc = """ PICO_STATUS UsbDrDaqSetPWM
    (
	    int16_t    handle,
		USB_DRDAQ_GPIO    IOChannel,
		uint16_t    period,
		uint8_t    cycle
	); """
usbDrDaq.make_symbol("_SetPWM", "UsbDrDaqSetPWM", c_uint32, [c_int16, c_uint32, c_uint16, c_uint8], doc)

doc = """ PICO_STATUS UsbDrDaqSetRGBLED
    (
	    int16_t    handle,
		uint16_t    red,
		uint16_t    green,
		uint16_t    blue
	); """
usbDrDaq.make_symbol("_SetRGBLED", "UsbDrDaqSetRGBLED", c_uint32, [c_int16, c_uint16, c_uint16, c_uint16], doc)

doc = """ PICO_STATUS UsbDrDaqSetScalings
    (
	    int16_t    handle,
		USB_DRDAQ_INPUTS    channel,
		int16_t    scalingNumber
	); """
usbDrDaq.make_symbol("_SetScalings", "UsbDrDaqSetScalings", c_uint32, [c_int16, c_uint32, c_int16], doc)

doc = """ PICO_STATUS UsbDrDaqSetSigGenArbitary
    (
	    int16_t    handle,
		int32_t    offsetVoltage,
		uint32_t    pkToPk,
		int16_t    *arbitaryWaveform,
		int16_t    arbitaryWaveformSize,
		int32_t    updateRate
	); """
usbDrDaq.make_symbol("_SetSigGenArbitary", "UsbDrDaqSetSigGenArbitrary", c_uint32, [c_int16, c_int32, c_uint32, c_void_p, c_int16, c_int32], doc)

doc = """ PICO_STATUS UsbDrDaqSetSigGenBuiltIn
    (
	    int16_t    handle,
		int32_t    offsetVoltage,
		uint32_t    pkToPk,
		int16_t    frequency,
		USB_DRDAQ_WAVE    waveType
	); """
usbDrDaq.make_symbol("_SetSigGenBuiltIn", "UsbDrDaqSetSigGenBuiltIn", c_uint32, [c_int16, c_int32, c_uint32, c_int16, c_uint32], doc)

doc = """ PICO_STATUS UsbDrDaqSetTrigger
    (
	    int16_t    handle,
		uint16_t    enabled,
		uint16_t    auto_trigger,
		uint16_t    auto_ms,
		uint16_t    channel,
		uint16_t    dir,
		int16_t    threshold,
		uint16_t    hysteresis,
		float    delay
	); """
usbDrDaq.make_symbol("_SetTrigger", "UsbDrDaqSetTrigger", c_uint32, [c_int16, c_uint16, c_uint16, c_uint16, c_uint16, c_uint16,c_int16, c_uint16, c_float], doc)

doc = """ PICO_STATUS UsbDrDaqStartPulseCount
    (
	    int16_t    handle,
		USB_DRDAQ_GPIO    IOChannel,
		int16_t    direction
	); """
usbDrDaq.make_symbol("_StartPulseCount", "UsbDrDaqStartPulseCount", c_uint32, [c_int16, c_uint32, c_int16], doc)

doc = """ PICO_STATUS UsbDrDaqStop
    (
	    int16_t    handle
	); """
usbDrDaq.make_symbol("_Stop", "UsbDrDaqStop", c_uint32, [c_int16], doc)

doc = """ PICO_STATUS UsbDrDaqStopSigGen
    (
	    int16_t    handle
	); """
usbDrDaq.make_symbol("_StopSigGen", "UsbDrDaqStopSigGen", c_uint32, [c_int16], doc)


from ctypes import POINTER, c_int16, c_uint32

import matplotlib.pyplot as plt
import numpy as np

from picosdk.ps2000 import ps2000
from picosdk.functions import assert_pico2000_ok
from picosdk.PicoDeviceEnums import picoEnum
from picosdk.ctypes_wrapper import C_CALLBACK_FUNCTION_FACTORY

import time

CALLBACK = C_CALLBACK_FUNCTION_FACTORY(
    None,
    POINTER(POINTER(c_int16)),
    c_int16,
    c_uint32,
    c_int16,
    c_int16,
    c_uint32
)

adc_values = []


def get_overview_buffers(buffers, _overflow, _triggered_at, _triggered, _auto_stop, n_values):
    adc_values.extend(buffers[0][0:n_values])


callback = CALLBACK(get_overview_buffers)


def adc_to_mv(values, range_, bitness=16):
    v_ranges = [10, 20, 50, 100, 200, 500, 1_000, 2_000, 5_000, 10_000, 20_000]

    return [(x * v_ranges[range_]) / (2**(bitness - 1) - 1) for x in values]


with ps2000.open_unit() as device:
    print('Device info: {}'.format(device.info))

    res = ps2000.ps2000_set_channel(
        device.handle,
        picoEnum.PICO_CHANNEL['PICO_CHANNEL_A'],
        True,
        picoEnum.PICO_COUPLING['PICO_DC'],
        ps2000.PS2000_VOLTAGE_RANGE['PS2000_50MV'],
    )
    assert_pico2000_ok(res)

    res = ps2000.ps2000_run_streaming_ns(
        device.handle,
        500,
        2,
        100_000,
        False,
        1,
        50_000
    )
    assert_pico2000_ok(res)

    start_time = time.time_ns()

    while time.time_ns() - start_time < 60_000_000:
        ps2000.ps2000_get_streaming_last_values(
            device.handle,
            callback
        )

    end_time = time.time_ns()

    ps2000.ps2000_stop(device.handle)

    mv_values = adc_to_mv(adc_values, ps2000.PS2000_VOLTAGE_RANGE['PS2000_50MV'])

    fig, ax = plt.subplots()

    ax.set_xlabel('time/ms')
    ax.set_ylabel('voltage/mV')
    ax.plot(np.linspace(0, (end_time - start_time) * 1e-6, len(mv_values)), mv_values)

    plt.show()



#
# Copyright (C) 2018 Pico Technology Ltd. See LICENSE file for terms.
#
# PS4000A RAPID BLOCK MODE EXAMPLE
# This example opens a 4000a driver device, sets up two channels and a trigger then collects 10 blocks of data with additional driver calls.
# This data is then plotted as mV against time in ns.

import ctypes
import numpy as np
from picosdk.ps4000a import ps4000a as ps
import matplotlib.pyplot as plt
from picosdk.functions import adc2mV, assert_pico_ok

# Create chandle and status ready for use
chandle = ctypes.c_int16()
status = {}

# Open 4000 series PicoScope
# Returns handle to chandle for use in future API functions
status["openunit"] = ps.ps4000aOpenUnit(ctypes.byref(chandle), None)

try:
    assert_pico_ok(status["openunit"])
except: # PicoNotOkError:
    powerStatus = status["openunit"]

    if powerStatus == 286:
        status["changePowerSource"] = ps.ps4000aChangePowerSource(chandle, powerStatus)
    elif powerStatus == 282:
        status["changePowerSource"] = ps.ps4000aChangePowerSource(chandle, powerStatus)
    else:
        raise

    assert_pico_ok(status["changePowerSource"])

# Set up channel A
# handle = chandle
# channel = PS4000a_CHANNEL_A = 0
# enabled = 1
# coupling type = PS4000a_DC = 1
# range = PS4000a_2V = 7
# analogOffset = 0 V
chARange = 7
status["setChA"] = ps.ps4000aSetChannel(chandle, 0, 1, 1, chARange, 0)
assert_pico_ok(status["setChA"])

# Set up channel B
# handle = chandle
# channel = PS4000a_CHANNEL_B = 1
# enabled = 1
# coupling type = PS4000a_DC = 1
# range = PS4000a_2V = 7
# analogOffset = 0 V
chBRange = 7
status["setChB"] = ps.ps4000aSetChannel(chandle, 1, 1, 1, chBRange, 0)
assert_pico_ok(status["setChB"])

# Set up single trigger
# handle = chandle
# enabled = 1
# source = PS4000a_CHANNEL_A = 0
# threshold = 1024 ADC counts
# direction = PS4000a_RISING = 2
# delay = 0 s
# auto Trigger = 1000 ms
status["trigger"] = ps.ps4000aSetSimpleTrigger(chandle, 1, 0, 1024, 2, 0, 1000)
assert_pico_ok(status["trigger"])

# Set number of pre and post trigger samples to be collected
preTriggerSamples = 2500
postTriggerSamples = 2500
maxSamples = preTriggerSamples + postTriggerSamples

# Get timebase information
# WARNING: When using this example it may not be possible to access all Timebases as all channels are enabled by default when opening the scope.  
# To access these Timebases, set any unused analogue channels to off.
# handle = chandle
# timebase = 8 = timebase
# noSamples = maxSamples
# pointer to timeIntervalNanoseconds = ctypes.byref(timeIntervalns)
# pointer to maxSamples = ctypes.byref(returnedMaxSamples)
# segment index = 0
timebase = 8
timeIntervalns = ctypes.c_float()
returnedMaxSamples = ctypes.c_int32()
oversample = ctypes.c_int16(1)
status["getTimebase2"] = ps.ps4000aGetTimebase2(chandle, timebase, maxSamples, ctypes.byref(timeIntervalns), ctypes.byref(returnedMaxSamples), 0)
assert_pico_ok(status["getTimebase2"])

# Set memory segments
# handle = chandle
# nSegments = 10
nMaxSamples = ctypes.c_int32(0)
status["setMemorySegments"] = ps.ps4000aMemorySegments(chandle, 10, ctypes.byref(nMaxSamples))
assert_pico_ok(status["setMemorySegments"])

# Set number of captures
# handle = chandle
# nCaptures = 10
status["SetNoOfCaptures"] = ps.ps4000aSetNoOfCaptures(chandle, 10)
assert_pico_ok(status["SetNoOfCaptures"])

# set up buffers
bufferA0 = (ctypes.c_int16 * maxSamples)()
bufferA1 = (ctypes.c_int16 * maxSamples)()
bufferA2 = (ctypes.c_int16 * maxSamples)()
bufferA3 = (ctypes.c_int16 * maxSamples)()
bufferA4 = (ctypes.c_int16 * maxSamples)()
bufferA5 = (ctypes.c_int16 * maxSamples)()
bufferA6 = (ctypes.c_int16 * maxSamples)()
bufferA7 = (ctypes.c_int16 * maxSamples)()
bufferA8 = (ctypes.c_int16 * maxSamples)()
bufferA9 = (ctypes.c_int16 * maxSamples)()

bufferB0 = (ctypes.c_int16 * maxSamples)()
bufferB1 = (ctypes.c_int16 * maxSamples)()
bufferB2 = (ctypes.c_int16 * maxSamples)()
bufferB3 = (ctypes.c_int16 * maxSamples)()
bufferB4 = (ctypes.c_int16 * maxSamples)()
bufferB5 = (ctypes.c_int16 * maxSamples)()
bufferB6 = (ctypes.c_int16 * maxSamples)()
bufferB7 = (ctypes.c_int16 * maxSamples)()
bufferB8 = (ctypes.c_int16 * maxSamples)()
bufferB9 = (ctypes.c_int16 * maxSamples)()

# set data buffers
# handle = chandle
# channel = PS4000A_CHANNEL_A = 0
# buffer = bufferAX (where X = segmentIndex)
# bufferLength = maxSamples
# segmentIndex = X
# mode = PS4000A_RATIO_MODE_NONE = 0

status["setDataBufferA0"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferA0), maxSamples, 0, 0)
status["setDataBufferA1"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferA1), maxSamples, 1, 0)
status["setDataBufferA2"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferA2), maxSamples, 2, 0)
status["setDataBufferA3"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferA3), maxSamples, 3, 0)
status["setDataBufferA4"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferA4), maxSamples, 4, 0)
status["setDataBufferA5"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferA5), maxSamples, 5, 0)
status["setDataBufferA6"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferA6), maxSamples, 6, 0)
status["setDataBufferA7"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferA7), maxSamples, 7, 0)
status["setDataBufferA8"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferA8), maxSamples, 8, 0)
status["setDataBufferA9"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferA9), maxSamples, 9, 0)

status["setDataBufferB0"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferB0), maxSamples, 0, 0)
status["setDataBufferB1"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferB1), maxSamples, 1, 0)
status["setDataBufferB2"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferB2), maxSamples, 2, 0)
status["setDataBufferB3"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferB3), maxSamples, 3, 0)
status["setDataBufferB4"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferB4), maxSamples, 4, 0)
status["setDataBufferB5"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferB5), maxSamples, 5, 0)
status["setDataBufferB6"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferB6), maxSamples, 6, 0)
status["setDataBufferB7"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferB7), maxSamples, 7, 0)
status["setDataBufferB8"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferB8), maxSamples, 8, 0)
status["setDataBufferB9"] = ps.ps4000aSetDataBuffer(chandle, 0, ctypes.byref(bufferB9), maxSamples, 9, 0)

# run block capture
# handle = chandle
# number of pre-trigger samples = preTriggerSamples
# number of post-trigger samples = PostTriggerSamples
# timebase = 3 = 80 ns = timebase (see Programmer's guide for mre information on timebases)
# time indisposed ms = None (not needed in the example)
# segment index = 0
# lpReady = None (using ps4000aIsReady rather than ps4000aBlockReady)
# pParameter = None
status["runBlock"] = ps.ps4000aRunBlock(chandle, preTriggerSamples, postTriggerSamples, timebase, None, 0, None, None)
assert_pico_ok(status["runBlock"])

# check for end of capture
ready = ctypes.c_int16(0)
check = ctypes.c_int16(0)
while ready.value == check.value:
    status["isReady"] = ps.ps4000aIsReady(chandle, ctypes.byref(ready))

# Creates a overlow location for data
overflow = (ctypes.c_int16 * 10)()
# Creates converted types maxsamples
cmaxSamples = ctypes.c_int32(maxSamples)

# collect data 
# handle = chandle
# noOfSamples = cmaxSamples
# fromSegmentIndex = 0
# toSegmentIndex = 9
# downSampleRatio = 1
# downSampleRatioMode = PS4000A_RATIO_MODE_NONE
status["getValuesBulk"] = ps.ps4000aGetValuesBulk(chandle, ctypes.byref(cmaxSamples), 0, 9, 1, 0, ctypes.byref(overflow))
assert_pico_ok(status["getValuesBulk"])

# find maximum ADC count value
# handle = chandle
# pointer to value = ctypes.byref(maxADC)
maxADC = ctypes.c_int16(32767)

# convert from adc to mV
adc2mVChA0 =  adc2mV(bufferA0, chARange, maxADC)
adc2mVChA1 =  adc2mV(bufferA1, chARange, maxADC)
adc2mVChA2 =  adc2mV(bufferA2, chARange, maxADC)
adc2mVChA3 =  adc2mV(bufferA3, chARange, maxADC)
adc2mVChA4 =  adc2mV(bufferA4, chARange, maxADC)
adc2mVChA5 =  adc2mV(bufferA5, chARange, maxADC)
adc2mVChA6 =  adc2mV(bufferA6, chARange, maxADC)
adc2mVChA7 =  adc2mV(bufferA7, chARange, maxADC)
adc2mVChA8 =  adc2mV(bufferA8, chARange, maxADC)
adc2mVChA9 =  adc2mV(bufferA9, chARange, maxADC)

adc2mVChB0 =  adc2mV(bufferB0, chARange, maxADC)
adc2mVChB1 =  adc2mV(bufferB1, chARange, maxADC)
adc2mVChB2 =  adc2mV(bufferB2, chARange, maxADC)
adc2mVChB3 =  adc2mV(bufferB3, chARange, maxADC)
adc2mVChB4 =  adc2mV(bufferB4, chARange, maxADC)
adc2mVChB5 =  adc2mV(bufferB5, chARange, maxADC)
adc2mVChB6 =  adc2mV(bufferB6, chARange, maxADC)
adc2mVChB7 =  adc2mV(bufferB7, chARange, maxADC)
adc2mVChB8 =  adc2mV(bufferB8, chARange, maxADC)
adc2mVChB9 =  adc2mV(bufferB9, chARange, maxADC)

# Create time data
time = np.linspace(0, ((cmaxSamples.value)-1) * timeIntervalns.value, cmaxSamples.value)

# plot data
plt.plot(time, adc2mVChA0)
plt.plot(time, adc2mVChA1)
plt.plot(time, adc2mVChA2)
plt.plot(time, adc2mVChA3)
plt.plot(time, adc2mVChA4)
plt.plot(time, adc2mVChA5)
plt.plot(time, adc2mVChA6)
plt.plot(time, adc2mVChA7)
plt.plot(time, adc2mVChA8)
plt.plot(time, adc2mVChA9)
plt.xlabel('Time (ns)')
plt.ylabel('Voltage (mV)')
plt.show()

# Stop the scope
# handle = chandle
status["stop"] = ps.ps4000aStop(chandle)
assert_pico_ok(status["stop"])

# Close unitDisconnect the scope
# handle = chandle
status["close"] = ps.ps4000aCloseUnit(chandle)
assert_pico_ok(status["close"])

# display status returns
print(status)

